"""Users and groups.
"""
__docformat__ = 'restructuredtext en'


from utils import *
from enums import *


class User(Cached):
    """Represents a Skype user.
    """
    _ValidateHandle = str

    def __repr__(self):
        return Cached.__repr__(self, 'Handle')

    def _Property(self, PropName, Set=None, Cache=True):
        return self._Owner._Property('USER', self.Handle, PropName, Set, Cache)

    def SaveAvatarToFile(self, Filename, AvatarId=1):
        """Saves user avatar to a file.

        :Parameters:
          Filename : str
            Destination path.
          AvatarId : int
            Avatar Id.
        """
        s = 'USER %s AVATAR %s %s' % (self.Handle, AvatarId, path2unicode(Filename))
        self._Owner._DoCommand('GET %s' % s, s)

    def SetBuddyStatusPendingAuthorization(self, Text=u''):
        """Sets the BuddyStaus property to `enums.budPendingAuthorization`
        additionally specifying the authorization text.

        :Parameters:
          Text : unicode
            The authorization text.

        :see: `BuddyStatus`
        """
        self._Property('BUDDYSTATUS', '%d %s' % (budPendingAuthorization, tounicode(Text)), Cache=False)

    def _GetAbout(self):
        return self._Property('ABOUT')

    About = property(_GetAbout,
    doc="""About text of the user.

    :type: unicode
    """)

    def _GetAliases(self):
        return split(self._Property('ALIASES'))

    Aliases = property(_GetAliases,
    doc="""Aliases of the user.

    :type: list of str
    """)

    def _GetBirthday(self):
        value = self._Property('BIRTHDAY')
        if len(value) == 8:
            from datetime import date
            from time import strptime
            return date(*strptime(value, '%Y%m%d')[:3])

    Birthday = property(_GetBirthday,
    doc="""Birthday of the user. None if not set.

    :type: datetime.date or None
    """)

    def _GetBuddyStatus(self):
        return int(self._Property('BUDDYSTATUS'))

    def _SetBuddyStatus(self, Value):
        self._Property('BUDDYSTATUS', int(Value), Cache=False)

    BuddyStatus = property(_GetBuddyStatus, _SetBuddyStatus,
    doc="""Buddy status of the user.

    :type: `enums`.bud*
    """)

    def _GetCanLeaveVoicemail(self):
        return (self._Property('CAN_LEAVE_VM') == 'TRUE')

    CanLeaveVoicemail = property(_GetCanLeaveVoicemail,
    doc="""Tells if it is possible to send voicemail to the user.

    :type: bool
    """)

    def _GetCity(self):
        return self._Property('CITY')

    City = property(_GetCity,
    doc="""City of the user.

    :type: unicode
    """)

    def _GetCountry(self):
        value = self._Property('COUNTRY')
        if value:
            if self._Owner.Protocol >= 4:
                value = chop(value)[-1]
        return value

    Country = property(_GetCountry,
    doc="""Country of the user.

    :type: unicode
    """)

    def _GetCountryCode(self):
        if self._Owner.Protocol < 4:
            return ''
        value = self._Property('COUNTRY')
        if value:
            value = chop(value)[0]
        return str(value)

    CountryCode = property(_GetCountryCode,
    doc="""ISO country code of the user.

    :type: str
    """)

    def _GetDisplayName(self):
        return self._Property('DISPLAYNAME')

    def _SetDisplayName(self, Value):
        self._Property('DISPLAYNAME', Value)

    DisplayName = property(_GetDisplayName, _SetDisplayName,
    doc="""Display name of the user.

    :type: unicode
    """)

    def _GetHandle(self):
        return self._Handle

    Handle = property(_GetHandle,
    doc="""Skypename of the user.

    :type: str
    """)

    def _GetFullName(self):
        return self._Property('FULLNAME')

    FullName = property(_GetFullName,
    doc="""Full name of the user.

    :type: unicode
    """)

    def _GetHasCallEquipment(self):
        return self._Property('HASCALLEQUIPMENT') == 'TRUE'

    HasCallEquipment = property(_GetHasCallEquipment,
    doc="""Tells if the user has call equipment.

    :type: bool
    """)

    def _GetHomepage(self):
        return self._Property('HOMEPAGE')

    Homepage = property(_GetHomepage,
    doc="""Homepage URL of the user.

    :type: unicode
    """)

    def _GetIsAuthorized(self):
        return (self._Property('ISAUTHORIZED') == 'TRUE')

    def _SetIsAuthorized(self, Value):
        self._Property('ISAUTHORIZED', cndexp(Value, 'TRUE', 'FALSE'))

    IsAuthorized = property(_GetIsAuthorized, _SetIsAuthorized,
    doc="""Tells if the user is authorized to contact us.

    :type: bool
    """)

    def _GetIsBlocked(self):
        return (self._Property('ISBLOCKED') == 'TRUE')

    def _SetIsBlocked(self, Value):
        self._Property('ISBLOCKED', cndexp(Value, 'TRUE', 'FALSE'))

    IsBlocked = property(_GetIsBlocked, _SetIsBlocked,
    doc="""Tells whether this user is blocked or not.

    :type: bool
    """)

    def _GetIsCallForwardActive(self):
        return (self._Property('IS_CF_ACTIVE') == 'TRUE')

    IsCallForwardActive = property(_GetIsCallForwardActive,
    doc="""Tells whether the user has Call Forwarding activated or not.

    :type: bool
    """)

    def _GetIsSkypeOutContact(self):
        return (self.OnlineStatus == olsSkypeOut)

    IsSkypeOutContact = property(_GetIsSkypeOutContact,
    doc="""Tells whether a user is a SkypeOut contact.

    :type: bool
    """)

    def _GetIsVideoCapable(self):
        return (self._Property('IS_VIDEO_CAPABLE') == 'TRUE')

    IsVideoCapable = property(_GetIsVideoCapable,
    doc="""Tells if the user has video capability.

    :type: bool
    """)

    def _GetIsVoicemailCapable(self):
        return (self._Property('IS_VOICEMAIL_CAPABLE') == 'TRUE')

    IsVoicemailCapable = property(_GetIsVoicemailCapable,
    doc="""Tells if the user has voicemail capability.

    :type: bool
    """)

    def _GetLanguage(self):
        value = self._Property('LANGUAGE')
        if value:
            if self._Owner.Protocol >= 4:
                value = chop(value)[-1]
        return value

    Language = property(_GetLanguage,
    doc="""The language of the user.

    :type: unicode
    """)

    def _GetLanguageCode(self):
        if self._Owner.Protocol < 4:
            return u''
        value = self._Property('LANGUAGE')
        if value:
            value = chop(value)[0]
        return str(value)

    LanguageCode = property(_GetLanguageCode,
    doc="""The ISO language code of the user.

    :type: str
    """)

    def _GetLastOnline(self):
        return float(self._Property('LASTONLINETIMESTAMP'))

    LastOnline = property(_GetLastOnline,
    doc="""The time when a user was last online as a timestamp.

    :type: float

    :see: `LastOnlineDatetime`
    """)

    def _GetLastOnlineDatetime(self):
        from datetime import datetime
        return datetime.fromtimestamp(self.LastOnline)

    LastOnlineDatetime = property(_GetLastOnlineDatetime,
    doc="""The time when a user was last online as a datetime.

    :type: datetime.datetime

    :see: `LastOnline`
    """)

    def _GetMoodText(self):
        return self._Property('MOOD_TEXT')

    MoodText = property(_GetMoodText,
    doc="""Mood text of the user.

    :type: unicode
    """)

    def _GetNumberOfAuthBuddies(self):
        return int(self._Property('NROF_AUTHED_BUDDIES'))

    NumberOfAuthBuddies = property(_GetNumberOfAuthBuddies,
    doc="""Number of authenticated buddies in user's contact list.

    :type: int
    """)

    def _GetOnlineStatus(self):
        return str(self._Property('ONLINESTATUS'))

    OnlineStatus = property(_GetOnlineStatus,
    doc="""Online status of the user.

    :type: `enums`.ols*
    """)

    def _GetPhoneHome(self):
        return self._Property('PHONE_HOME')

    PhoneHome = property(_GetPhoneHome,
    doc="""Home telephone number of the user.

    :type: unicode
    """)

    def _GetPhoneMobile(self):
        return self._Property('PHONE_MOBILE')

    PhoneMobile = property(_GetPhoneMobile,
    doc="""Mobile telephone number of the user.

    :type: unicode
    """)

    def _GetPhoneOffice(self):
        return self._Property('PHONE_OFFICE')

    PhoneOffice = property(_GetPhoneOffice,
    doc="""Office telephone number of the user.

    :type: unicode
    """)

    def _GetProvince(self):
        return self._Property('PROVINCE')

    Province = property(_GetProvince,
    doc="""Province of the user.

    :type: unicode
    """)

    def _GetReceivedAuthRequest(self):
        return self._Property('RECEIVEDAUTHREQUEST')

    ReceivedAuthRequest = property(_GetReceivedAuthRequest,
    doc="""Text message for authorization request. Available only when user asks for authorization.

    :type: unicode
    """)

    def _GetRichMoodText(self):
        return self._Property('RICH_MOOD_TEXT')

    RichMoodText = property(_GetRichMoodText,
    doc="""Advanced version of `MoodText`.

    :type: unicode

    :see: https://developer.skype.com/Docs/ApiDoc/SET_PROFILE_RICH_MOOD_TEXT
    """)

    def _GetSex(self):
        return str(self._Property('SEX'))

    Sex = property(_GetSex,
    doc="""Sex of the user.

    :type: `enums`.usex*
    """)

    def _GetSpeedDial(self):
        return self._Property('SPEEDDIAL')

    def _SetSpeedDial(self, Value):
        self._Property('SPEEDDIAL', Value)

    SpeedDial = property(_GetSpeedDial, _SetSpeedDial,
    doc="""Speed-dial code assigned to the user.

    :type: unicode
    """)

    def _GetTimezone(self):
        return int(self._Property('TIMEZONE'))

    Timezone = property(_GetTimezone,
    doc="""Timezone of the user in minutes from GMT.

    :type: int
    """)


class UserCollection(CachedCollection):
    _CachedType = User


class Group(Cached):
    """Represents a group of Skype users.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id')

    def _Alter(self, AlterName, Args=None):
        return self._Owner._Alter('GROUP', self.Id, AlterName, Args)

    def _Property(self, PropName, Value=None, Cache=True):
        return self._Owner._Property('GROUP', self.Id, PropName, Value, Cache)

    def Accept(self):
        """Accepts an invitation to join a shared contact group.
        """
        self._Alter('ACCEPT')

    def AddUser(self, Username):
        """Adds new a user to the group.

        :Parameters:
          Username : str
            Skypename of the new user.
        """
        self._Alter('ADDUSER', Username)

    def Decline(self):
        """Declines an invitation to join a shared contact group.
        """
        self._Alter('DECLINE')

    def RemoveUser(self, Username):
        """Removes a user from the group.

        :Parameters:
          Username : str
            Skypename of the user.
        """
        self._Alter('REMOVEUSER', Username)

    def Share(self, MessageText=''):
        """Shares a contact group.

        :Parameters:
          MessageText : unicode
            Message text for group members.
        """
        self._Alter('SHARE', MessageText)

    def _GetCustomGroupId(self):
        return str(self._Property('CUSTOM_GROUP_ID'))

    CustomGroupId = property(_GetCustomGroupId,
    doc="""Persistent group ID. The custom group ID is a persistent value that does not change.

    :type: str
    """)

    def _GetDisplayName(self):
        return self._Property('DISPLAYNAME')

    def _SetDisplayName(self, Value):
        self._Property('DISPLAYNAME', Value)

    DisplayName = property(_GetDisplayName, _SetDisplayName,
    doc="""Display name of the group.

    :type: unicode
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""Group Id.

    :type: int
    """)

    def _GetIsExpanded(self):
        return self._Property('EXPANDED') == 'TRUE'

    IsExpanded = property(_GetIsExpanded,
    doc="""Tells if the group is expanded in the client.

    :type: bool
    """)

    def _GetIsVisible(self):
        return self._Property('VISIBLE') == 'TRUE'

    IsVisible = property(_GetIsVisible,
    doc="""Tells if the group is visible in the client.

    :type: bool
    """)

    def _GetOnlineUsers(self):
        return UserCollection(self._Owner, (x.Handle for x in self.Users if x.OnlineStatus == olsOnline))

    OnlineUsers = property(_GetOnlineUsers,
    doc="""Users of the group that are online

    :type: `UserCollection`
    """)

    def _GetType(self):
        return str(self._Property('TYPE'))

    Type = property(_GetType,
    doc="""Group type.

    :type: `enums`.grp*
    """)

    def _GetUsers(self):
        return UserCollection(self._Owner, split(self._Property('USERS', Cache=False), ', '))

    Users = property(_GetUsers,
    doc="""Users in this group.

    :type: `UserCollection`
    """)


class GroupCollection(CachedCollection):
    _CachedType = Group



"""Main Skype interface.
"""
__docformat__ = 'restructuredtext en'


import threading
import weakref
import logging

from api import *
from errors import *
from enums import *
from utils import *
from conversion import *
from client import *
from user import *
from call import *
from profile import *
from settings import *
from chat import *
from application import *
from voicemail import *
from sms import *
from filetransfer import *


class APINotifier(SkypeAPINotifier):
    def __init__(self, skype):
        self.skype = weakref.proxy(skype)

    def attachment_changed(self, status):
        try:
            self.skype._CallEventHandler('AttachmentStatus', status)
            if status == apiAttachRefused:
                raise SkypeAPIError('Skype connection refused')
        except weakref.ReferenceError:
            pass

    def notification_received(self, notification):
        try:
            skype = self.skype
            skype._CallEventHandler('Notify', notification)
            a, b = chop(notification)
            object_type = None
            # if..elif handling cache and most event handlers
            if a in ('CALL', 'USER', 'GROUP', 'CHAT', 'CHATMESSAGE', 'CHATMEMBER', 'VOICEMAIL', 'APPLICATION', 'SMS', 'FILETRANSFER'):
                object_type, object_id, prop_name, value = [a] + chop(b, 2)
                skype._CacheDict[str(object_type), str(object_id), str(prop_name)] = value
                if object_type == 'USER':
                    o = User(skype, object_id)
                    if prop_name == 'ONLINESTATUS':
                        skype._CallEventHandler('OnlineStatus', o, str(value))
                    elif prop_name == 'MOOD_TEXT' or prop_name == 'RICH_MOOD_TEXT':
                        skype._CallEventHandler('UserMood', o, value)
                    elif prop_name == 'RECEIVEDAUTHREQUEST':
                        skype._CallEventHandler('UserAuthorizationRequestReceived', o)
                elif object_type == 'CALL':
                    o = Call(skype, object_id)
                    if prop_name == 'STATUS':
                        skype._CallEventHandler('CallStatus', o, str(value))
                    elif prop_name == 'SEEN':
                        skype._CallEventHandler('CallSeenStatusChanged', o, (value == 'TRUE'))
                    elif prop_name == 'VAA_INPUT_STATUS':
                        skype._CallEventHandler('CallInputStatusChanged', o, (value == 'TRUE'))
                    elif prop_name == 'TRANSFER_STATUS':
                        skype._CallEventHandler('CallTransferStatusChanged', o, str(value))
                    elif prop_name == 'DTMF':
                        skype._CallEventHandler('CallDtmfReceived', o, str(value))
                    elif prop_name == 'VIDEO_STATUS':
                        skype._CallEventHandler('CallVideoStatusChanged', o, str(value))
                    elif prop_name == 'VIDEO_SEND_STATUS':
                        skype._CallEventHandler('CallVideoSendStatusChanged', o, str(value))
                    elif prop_name == 'VIDEO_RECEIVE_STATUS':
                        skype._CallEventHandler('CallVideoReceiveStatusChanged', o, str(value))
                elif object_type == 'CHAT':
                    o = Chat(skype, object_id)
                    if prop_name == 'MEMBERS':
                        skype._CallEventHandler('ChatMembersChanged', o, UserCollection(skype, split(value)))
                    if prop_name in ('OPENED', 'CLOSED'):
                        skype._CallEventHandler('ChatWindowState', o, (prop_name == 'OPENED'))
                elif object_type == 'CHATMEMBER':
                    o = ChatMember(skype, object_id)
                    if prop_name == 'ROLE':
                        skype._CallEventHandler('ChatMemberRoleChanged', o, str(value))
                elif object_type == 'CHATMESSAGE':
                    o = ChatMessage(skype, object_id)
                    if prop_name == 'STATUS':
                        skype._CallEventHandler('MessageStatus', o, str(value))
                elif object_type == 'APPLICATION':
                    o = Application(skype, object_id)
                    if prop_name == 'CONNECTING':
                        skype._CallEventHandler('ApplicationConnecting', o, UserCollection(skype, split(value)))
                    elif prop_name == 'STREAMS':
                        skype._CallEventHandler('ApplicationStreams', o, ApplicationStreamCollection(o, split(value)))
                    elif prop_name == 'DATAGRAM':
                        handle, text = chop(value)
                        skype._CallEventHandler('ApplicationDatagram', o, ApplicationStream(o, handle), text)
                    elif prop_name == 'SENDING':
                        skype._CallEventHandler('ApplicationSending', o, ApplicationStreamCollection(o, (x.split('=')[0] for x in split(value))))
                    elif prop_name == 'RECEIVED':
                        skype._CallEventHandler('ApplicationReceiving', o, ApplicationStreamCollection(o, (x.split('=')[0] for x in split(value))))
                elif object_type == 'GROUP':
                    o = Group(skype, object_id)
                    if prop_name == 'VISIBLE':
                        skype._CallEventHandler('GroupVisible', o, (value == 'TRUE'))
                    elif prop_name == 'EXPANDED':
                        skype._CallEventHandler('GroupExpanded', o, (value == 'TRUE'))
                    elif prop_name == 'NROFUSERS':
                        skype._CallEventHandler('GroupUsers', o, int(value))
                elif object_type == 'SMS':
                    o = SmsMessage(skype, object_id)
                    if prop_name == 'STATUS':
                        skype._CallEventHandler('SmsMessageStatusChanged', o, str(value))
                    elif prop_name == 'TARGET_STATUSES':
                        for t in split(value, ', '):
                            number, status = t.split('=')
                            skype._CallEventHandler('SmsTargetStatusChanged', SmsTarget(o, number), str(status))
                elif object_type == 'FILETRANSFER':
                    o = FileTransfer(skype, object_id)
                    if prop_name == 'STATUS':
                        skype._CallEventHandler('FileTransferStatusChanged', o, str(value))
                elif object_type == 'VOICEMAIL':
                    o = Voicemail(skype, object_id)
                    if prop_name == 'STATUS':
                        skype._CallEventHandler('VoicemailStatus', o, str(value))
            elif a in ('PROFILE', 'PRIVILEGE'):
                object_type, object_id, prop_name, value = [a, ''] + chop(b)
                skype._CacheDict[str(object_type), str(object_id), str(prop_name)] = value
            elif a in ('CURRENTUSERHANDLE', 'USERSTATUS', 'CONNSTATUS', 'PREDICTIVE_DIALER_COUNTRY', 'SILENT_MODE', 'AUDIO_IN', 'AUDIO_OUT', 'RINGER', 'MUTE', 'AUTOAWAY', 'WINDOWSTATE'):
                object_type, object_id, prop_name, value = [a, '', '', b]
                skype._CacheDict[str(object_type), str(object_id), str(prop_name)] = value
                if object_type == 'MUTE':
                    skype._CallEventHandler('Mute', value == 'TRUE')
                elif object_type == 'CONNSTATUS':
                    skype._CallEventHandler('ConnectionStatus', str(value))
                elif object_type == 'USERSTATUS':
                    skype._CallEventHandler('UserStatus', str(value))
                elif object_type == 'AUTOAWAY':
                    skype._CallEventHandler('AutoAway', (value == 'ON'))
                elif object_type == 'WINDOWSTATE':
                    skype._CallEventHandler('ClientWindowState', str(value))
                elif object_type == 'SILENT_MODE':
                    skype._CallEventHandler('SilentModeStatusChanged', (value == 'ON'))
            elif a == 'CALLHISTORYCHANGED':
                skype._CallEventHandler('CallHistory')
            elif a == 'IMHISTORYCHANGED':
                skype._CallEventHandler('MessageHistory', '') # XXX: Arg is Skypename, which one?
            elif a == 'CONTACTS':
                prop_name, value = chop(b)
                if prop_name == 'FOCUSED':
                    skype._CallEventHandler('ContactsFocused', str(value))
            elif a == 'DELETED':
                prop_name, value = chop(b)
                if prop_name == 'GROUP':
                    skype._CallEventHandler('GroupDeleted', int(value))
            elif a == 'EVENT':
                object_id, prop_name, value = chop(b, 2)
                if prop_name == 'CLICKED':
                    skype._CallEventHandler('PluginEventClicked', PluginEvent(skype, object_id))
            elif a == 'MENU_ITEM':
                object_id, prop_name, value = chop(b, 2)
                if prop_name == 'CLICKED':
                    i = value.rfind('CONTEXT ')
                    if i >= 0:
                        context = chop(value[i+8:])[0]
                        users = ()
                        context_id = u''
                        if context in (pluginContextContact, pluginContextCall, pluginContextChat):
                            users = UserCollection(skype, split(value[:i-1], ', '))
                        if context in (pluginContextCall, pluginContextChat):
                            j = value.rfind('CONTEXT_ID ')
                            if j >= 0:
                                context_id = str(chop(value[j+11:])[0])
                                if context == pluginContextCall:
                                    context_id = int(context_id)
                        skype._CallEventHandler('PluginMenuItemClicked', PluginMenuItem(skype, object_id), users, str(context), context_id)
            elif a == 'WALLPAPER':
                skype._CallEventHandler('WallpaperChanged', unicode2path(b))
        except weakref.ReferenceError:
            pass

    def sending_command(self, command):
        try:
            self.skype._CallEventHandler('Command', command)
        except weakref.ReferenceError:
            pass

    def reply_received(self, command):
        try:
            self.skype._CallEventHandler('Reply', command)
        except weakref.ReferenceError:
            pass


class Skype(EventHandlingBase):
    """The main class which you have to instantiate to get access to the Skype client
    running currently in the background.

    Usage
    =====

       You should access this class using the alias at the package level:

       .. python::

           import Skype4Py

           skype = Skype4Py.Skype()

       Read the constructor (`Skype.__init__`) documentation for a list of accepted
       arguments.

    Events
    ======

       This class provides events.

       The events names and their arguments lists can be found in the `SkypeEvents`
       class in this module.

       The use of events is explained in the `EventHandlingBase` class
       which is a superclass of this class.
    """

    def __init__(self, Events=None, **Options):
        """Initializes the object.

        :Parameters:
          Events
            An optional object with event handlers. See `Skype4Py.utils.EventHandlingBase`
            for more information on events.
          Options
            Additional options for low-level API handler. See the `Skype4Py.api`
            subpackage for supported options. Available options may depend on the
            current platform. Note that the current platform can be queried using
            `Skype4Py.platform` variable.
        """
        self._Logger = logging.getLogger('Skype4Py.skype.Skype')
        self._Logger.info('object created')

        EventHandlingBase.__init__(self)
        if Events:
            self._SetEventHandlerObject(Events)

        try:
            self._Api = Options.pop('Api')
            if Options:
                raise TypeError('No options supported with custom API objects.')
        except KeyError:
            self._Api = SkypeAPI(Options)
        self._Api.set_notifier(APINotifier(self))

        Cached._CreateOwner(self)

        self._Cache = True
        self.ResetCache()

        from api import DEFAULT_TIMEOUT
        self._Timeout = DEFAULT_TIMEOUT

        self._Convert = Conversion(self)
        self._Client = Client(self)
        self._Settings = Settings(self)
        self._Profile = Profile(self)

    def __del__(self):
        """Frees all resources.
        """
        if hasattr(self, '_Api'):
            self._Api.close()

        self._Logger.info('object destroyed')

    def _DoCommand(self, Cmd, ExpectedReply=''):
        command = Command(Cmd, ExpectedReply, True, self.Timeout)
        self.SendCommand(command)
        a, b = chop(command.Reply)
        if a == 'ERROR':
            errnum, errstr = chop(b)
            self._CallEventHandler('Error', command, int(errnum), errstr)
            raise SkypeError(int(errnum), errstr)
        if not command.Reply.startswith(command.Expected):
            raise SkypeError(0, 'Unexpected reply from Skype, got [%s], expected [%s (...)]' % \
                (command.Reply, command.Expected))
        return command.Reply

    def _Property(self, ObjectType, ObjectId, PropName, Set=None, Cache=True):
        h = (str(ObjectType), str(ObjectId), str(PropName))
        arg = ('%s %s %s' % h).split()
        while '' in arg:
            arg.remove('')
        jarg = ' '.join(arg)
        if Set is None: # Get
            if Cache and self._Cache and h in self._CacheDict:
                return self._CacheDict[h]
            value = self._DoCommand('GET %s' % jarg, jarg)
            while arg:
                try:
                    a, b = chop(value)
                except ValueError:
                    break
                if a.lower() != arg[0].lower():
                    break
                del arg[0]
                value = b
            if Cache and self._Cache:
                self._CacheDict[h] = value
            return value
        else: # Set
            value = unicode(Set)
            self._DoCommand('SET %s %s' % (jarg, value), jarg)
            if Cache and self._Cache:
                self._CacheDict[h] = value

    def _Alter(self, ObjectType, ObjectId, AlterName, Args=None, Reply=None):
        cmd = 'ALTER %s %s %s' % (str(ObjectType), str(ObjectId), str(AlterName))
        if Reply is None:
            Reply = cmd
        if Args is not None:
            cmd = '%s %s' % (cmd, tounicode(Args))
        reply = self._DoCommand(cmd, Reply)
        arg = cmd.split()
        while arg:
            try:
                a, b = chop(reply)
            except ValueError:
                break
            if a.lower() != arg[0].lower():
                break
            del arg[0]
            reply = b
        return reply

    def _Search(self, ObjectType, Args=None):
        cmd = 'SEARCH %s' % ObjectType
        if Args is not None:
            cmd = '%s %s' % (cmd, Args)
        # It is safe to do str() as none of the searchable objects use non-ascii chars.
        return split(chop(str(self._DoCommand(cmd)))[-1], ', ')

    def ApiSecurityContextEnabled(self, Context):
        """Queries if an API security context for Internet Explorer is enabled.

        :Parameters:
          Context : unicode
            API security context to check.

        :return: True if the API security for the given context is enabled, False otherwise.
        :rtype: bool

        :warning: This functionality isn't supported by Skype4Py.
        """
        self._Api.security_context_enabled(Context)

    def Application(self, Name):
        """Queries an application object.

        :Parameters:
          Name : unicode
            Application name.

        :return: The application object.
        :rtype: `application.Application`
        """
        return Application(self, Name)

    def _AsyncSearchUsersReplyHandler(self, Command):
        if Command in self._AsyncSearchUsersCommands:
            self._AsyncSearchUsersCommands.remove(Command)
            self._CallEventHandler('AsyncSearchUsersFinished', Command.Id,
                UserCollection(self, split(chop(Command.Reply)[-1], ', ')))
            if len(self._AsyncSearchUsersCommands) == 0:
                self.UnregisterEventHandler('Reply', self._AsyncSearchUsersReplyHandler)
                del self._AsyncSearchUsersCommands

    def AsyncSearchUsers(self, Target):
        """Asynchronously searches for Skype users.

        :Parameters:
          Target : unicode
            Search target (name or email address).

        :return: A search identifier. It will be passed along with the results to the
                 `SkypeEvents.AsyncSearchUsersFinished` event after the search is completed.
        :rtype: int
        """
        if not hasattr(self, '_AsyncSearchUsersCommands'):
            self._AsyncSearchUsersCommands = []
            self.RegisterEventHandler('Reply', self._AsyncSearchUsersReplyHandler)
        command = Command('SEARCH USERS %s' % tounicode(Target), 'USERS', False, self.Timeout)
        self._AsyncSearchUsersCommands.append(command)
        self.SendCommand(command)
        # return pCookie - search identifier
        return command.Id

    def Attach(self, Protocol=5, Wait=True):
        """Establishes a connection to Skype.

        :Parameters:
          Protocol : int
            Minimal Skype protocol version.
          Wait : bool
            If set to False, blocks forever until the connection is established. Otherwise, timeouts
            after the `Timeout`.
        """
        try:
            self._Api.protocol = Protocol
            self._Api.attach(self.Timeout, Wait)
        except SkypeAPIError:
            self.ResetCache()
            raise

    def Call(self, Id=0):
        """Queries a call object.

        :Parameters:
          Id : int
            Call identifier.

        :return: Call object.
        :rtype: `call.Call`
        """
        o = Call(self, Id)
        o.Status # Test if such a call exists.
        return o

    def Calls(self, Target=''):
        """Queries calls in call history.

        :Parameters:
          Target : str
            Call target.

        :return: Call objects.
        :rtype: `CallCollection`
        """
        return CallCollection(self, self._Search('CALLS', Target))

    def _ChangeUserStatus_UserStatus(self, Status):
        if Status.upper() == self._ChangeUserStatus_Status:
            self._ChangeUserStatus_Event.set()

    def ChangeUserStatus(self, Status):
        """Changes the online status for the current user.

        :Parameters:
          Status : `enums`.cus*
            New online status for the user.

        :note: This function waits until the online status changes. Alternatively, use the
               `CurrentUserStatus` property to perform an immediate change of status.
        """
        if self.CurrentUserStatus.upper() == Status.upper():
            return
        self._ChangeUserStatus_Event = threading.Event()
        self._ChangeUserStatus_Status = Status.upper()
        self.RegisterEventHandler('UserStatus', self._ChangeUserStatus_UserStatus)
        self.CurrentUserStatus = Status
        self._ChangeUserStatus_Event.wait()
        self.UnregisterEventHandler('UserStatus', self._ChangeUserStatus_UserStatus)
        del self._ChangeUserStatus_Event, self._ChangeUserStatus_Status

    def Chat(self, Name=''):
        """Queries a chat object.

        :Parameters:
          Name : str
            Chat name.

        :return: A chat object.
        :rtype: `chat.Chat`
        """
        o = Chat(self, Name)
        o.Status # Tests if such a chat really exists.
        return o

    def ClearCallHistory(self, Username='ALL', Type=chsAllCalls):
        """Clears the call history.

        :Parameters:
          Username : str
            Skypename of the user. A special value of 'ALL' means that entries of all users should
            be removed.
          Type : `enums`.clt*
            Call type.
        """
        cmd = 'CLEAR CALLHISTORY %s %s' % (str(Type), Username)
        self._DoCommand(cmd, cmd)

    def ClearChatHistory(self):
        """Clears the chat history.
        """
        cmd = 'CLEAR CHATHISTORY'
        self._DoCommand(cmd, cmd)

    def ClearVoicemailHistory(self):
        """Clears the voicemail history.
        """
        self._DoCommand('CLEAR VOICEMAILHISTORY')

    def Command(self, Command, Reply=u'', Block=False, Timeout=30000, Id=-1):
        """Creates an API command object.

        :Parameters:
          Command : unicode
            Command string.
          Reply : unicode
            Expected reply. By default any reply is accepted (except errors which raise an
            `SkypeError` exception).
          Block : bool
            If set to True, `SendCommand` method waits for a response from Skype API before
            returning.
          Timeout : float, int or long
            Timeout. Used if Block == True. Timeout may be expressed in milliseconds if the type
            is int or long or in seconds (or fractions thereof) if the type is float.
          Id : int
            Command Id. The default (-1) means it will be assigned automatically as soon as the
            command is sent.

        :return: A command object.
        :rtype: `Command`

        :see: `SendCommand`
        """
        from api import Command as CommandClass
        return CommandClass(Command, Reply, Block, Timeout, Id)

    def Conference(self, Id=0):
        """Queries a call conference object.

        :Parameters:
          Id : int
            Conference Id.

        :return: A conference object.
        :rtype: `Conference`
        """
        o = Conference(self, Id)
        if Id <= 0 or not o.Calls:
            raise SkypeError(0, 'Unknown conference')
        return o

    def CreateChatUsingBlob(self, Blob):
        """Returns existing or joins a new chat using given blob.

        :Parameters:
          Blob : str
            A blob identifying the chat.

        :return: A chat object
        :rtype: `chat.Chat`
        """
        return Chat(self, chop(self._DoCommand('CHAT CREATEUSINGBLOB %s' % Blob), 2)[1])

    def CreateChatWith(self, *Usernames):
        """Creates a chat with one or more users.

        :Parameters:
          Usernames : str
            One or more Skypenames of the users.

        :return: A chat object
        :rtype: `Chat`

        :see: `Chat.AddMembers`
        """
        return Chat(self, chop(self._DoCommand('CHAT CREATE %s' % ', '.join(Usernames)), 2)[1])

    def CreateGroup(self, GroupName):
        """Creates a custom contact group.

        :Parameters:
          GroupName : unicode
            Group name.

        :return: A group object.
        :rtype: `Group`

        :see: `DeleteGroup`
        """
        groups = self.CustomGroups
        self._DoCommand('CREATE GROUP %s' % tounicode(GroupName))
        for g in self.CustomGroups:
            if g not in groups and g.DisplayName == GroupName:
                return g
        raise SkypeError(0, 'Group creating failed')

    def CreateSms(self, MessageType, *TargetNumbers):
        """Creates an SMS message.

        :Parameters:
          MessageType : `enums`.smsMessageType*
            Message type.
          TargetNumbers : str
            One or more target SMS numbers.

        :return: An sms message object.
        :rtype: `SmsMessage`
        """
        return SmsMessage(self, chop(self._DoCommand('CREATE SMS %s %s' % (MessageType, ', '.join(TargetNumbers))), 2)[1])

    def DeleteGroup(self, GroupId):
        """Deletes a custom contact group.

        Users in the contact group are moved to the All Contacts (hardwired) contact group.

        :Parameters:
          GroupId : int
            Group identifier. Get it from `Group.Id`.

        :see: `CreateGroup`
        """
        self._DoCommand('DELETE GROUP %s' % GroupId)

    def EnableApiSecurityContext(self, Context):
        """Enables an API security context for Internet Explorer scripts.

        :Parameters:
          Context : unicode
            combination of API security context values.

        :warning: This functionality isn't supported by Skype4Py.
        """
        self._Api.enable_security_context(Context)

    def FindChatUsingBlob(self, Blob):
        """Returns existing chat using given blob.

        :Parameters:
          Blob : str
            A blob identifying the chat.

        :return: A chat object
        :rtype: `chat.Chat`
        """
        return Chat(self, chop(self._DoCommand('CHAT FINDUSINGBLOB %s' % Blob), 2)[1])

    def Greeting(self, Username=''):
        """Queries the greeting used as voicemail.

        :Parameters:
          Username : str
            Skypename of the user.

        :return: A voicemail object.
        :rtype: `Voicemail`
        """
        for v in self.Voicemails:
            if Username and v.PartnerHandle != Username:
                continue
            if v.Type in (vmtDefaultGreeting, vmtCustomGreeting):
                return v

    def Message(self, Id=0):
        """Queries a chat message object.

        :Parameters:
          Id : int
            Message Id.

        :return: A chat message object.
        :rtype: `ChatMessage`
        """
        o = ChatMessage(self, Id)
        o.Status # Test if such an id is known.
        return o

    def Messages(self, Target=''):
        """Queries chat messages which were sent/received by the user.

        :Parameters:
          Target : str
            Message sender.

        :return: Chat message objects.
        :rtype: `ChatMessageCollection`
        """
        return ChatMessageCollection(self, self._Search('CHATMESSAGES', Target))

    def PlaceCall(self, *Targets):
        """Places a call to a single user or creates a conference call.

        :Parameters:
          Targets : str
            One or more call targets. If multiple targets are specified, a conference call is
            created. The call target can be a Skypename, phone number, or speed dial code.

        :return: A call object.
        :rtype: `call.Call`
        """
        calls = self.ActiveCalls
        reply = self._DoCommand('CALL %s' % ', '.join(Targets))
        # Skype for Windows returns the call status which gives us the call Id;
        if reply.startswith('CALL '):
            return Call(self, chop(reply, 2)[1])
        # On linux we get 'OK' as reply so we search for the new call on
        # list of active calls.
        for c in self.ActiveCalls:
            if c not in calls:
                return c
        raise SkypeError(0, 'Placing call failed')

    def Privilege(self, Name):
        """Queries the Skype services (privileges) enabled for the Skype client.

        :Parameters:
          Name : str
            Privilege name, currently one of 'SKYPEOUT', 'SKYPEIN', 'VOICEMAIL'.

        :return: True if the privilege is available, False otherwise.
        :rtype: bool
        """
        return (self._Property('PRIVILEGE', '', Name.upper()) == 'TRUE')

    def Profile(self, Property, Set=None):
        """Queries/sets user profile properties.

        :Parameters:
          Property : str
            Property name, currently one of 'PSTN_BALANCE', 'PSTN_BALANCE_CURRENCY', 'FULLNAME',
            'BIRTHDAY', 'SEX', 'LANGUAGES', 'COUNTRY', 'PROVINCE', 'CITY', 'PHONE_HOME',
            'PHONE_OFFICE', 'PHONE_MOBILE', 'HOMEPAGE', 'ABOUT'.
          Set : unicode or None
            Value the property should be set to or None if the value should be queried.

        :return: Property value if Set=None, None otherwise.
        :rtype: unicode or None
        """
        return self._Property('PROFILE', '', Property, Set)

    def Property(self, ObjectType, ObjectId, PropName, Set=None):
        """Queries/sets the properties of an object.

        :Parameters:
          ObjectType : str
            Object type ('USER', 'CALL', 'CHAT', 'CHATMESSAGE', ...).
          ObjectId : str
            Object Id, depends on the object type.
          PropName : str
            Name of the property to access.
          Set : unicode or None
            Value the property should be set to or None if the value should be queried.

        :return: Property value if Set=None, None otherwise.
        :rtype: unicode or None
        """
        return self._Property(ObjectType, ObjectId, PropName, Set)

    def ResetCache(self):
        """Deletes all command cache entries.

        This method clears the Skype4Py's internal command cache which means that all objects will forget
        their property values and querying them will trigger a code to get them from Skype client (and
        cache them again).
        """
        self._CacheDict = {}

    def SearchForUsers(self, Target):
        """Searches for users.

        :Parameters:
          Target : unicode
            Search target (name or email address).

        :return: Found users.
        :rtype: `UserCollection`
        """
        return UserCollection(self, self._Search('USERS', tounicode(Target)))

    def SendCommand(self, Command):
        """Sends an API command.

        :Parameters:
          Command : `Command`
            Command to send. Use `Command` method to create a command.
        """
        try:
            self._Api.send_command(Command)
        except SkypeAPIError:
            self.ResetCache()
            raise

    def SendMessage(self, Username, Text):
        """Sends a chat message.

        :Parameters:
          Username : str
            Skypename of the user.
          Text : unicode
            Body of the message.

        :return: A chat message object.
        :rtype: `ChatMessage`
        """
        return self.CreateChatWith(Username).SendMessage(Text)

    def SendSms(self, *TargetNumbers, **Properties):
        """Creates and sends an SMS message.

        :Parameters:
          TargetNumbers : str
            One or more target SMS numbers.
          Properties
            Message properties. Properties available are same as `SmsMessage` object properties.

        :return: An sms message object. The message is already sent at this point.
        :rtype: `SmsMessage`
        """
        sms = self.CreateSms(smsMessageTypeOutgoing, *TargetNumbers)
        for name, value in Properties.items():
            if isinstance(getattr(sms.__class__, name, None), property):
                setattr(sms, name, value)
            else:
                raise TypeError('Unknown property: %s' % prop)
        sms.Send()
        return sms

    def SendVoicemail(self, Username):
        """Sends a voicemail to a specified user.

        :Parameters:
          Username : str
            Skypename of the user.

        :note: Should return a `Voicemail` object. This is not implemented yet.
        """
        if self._Api.protocol >= 6:
            self._DoCommand('CALLVOICEMAIL %s' % Username)
        else:
            self._DoCommand('VOICEMAIL %s' % Username)

    def User(self, Username=''):
        """Queries a user object.

        :Parameters:
          Username : str
            Skypename of the user.

        :return: A user object.
        :rtype: `user.User`
        """
        if not Username:
            Username = self.CurrentUserHandle
        o = User(self, Username)
        o.OnlineStatus # Test if such a user exists.
        return o

    def Variable(self, Name, Set=None):
        """Queries/sets Skype general parameters.

        :Parameters:
          Name : str
            Variable name.
          Set : unicode or None
            Value the variable should be set to or None if the value should be queried.

        :return: Variable value if Set=None, None otherwise.
        :rtype: unicode or None
        """
        return self._Property(Name, '', '', Set)

    def Voicemail(self, Id):
        """Queries the voicemail object.

        :Parameters:
          Id : int
            Voicemail Id.

        :return: A voicemail object.
        :rtype: `Voicemail`
        """
        o = Voicemail(self, Id)
        o.Type # Test if such a voicemail exists.
        return o

    def _GetActiveCalls(self):
        return CallCollection(self, self._Search('ACTIVECALLS'))

    ActiveCalls = property(_GetActiveCalls,
    doc="""Queries a list of active calls.

    :type: `CallCollection`
    """)

    def _GetActiveChats(self):
        return ChatCollection(self, self._Search('ACTIVECHATS'))

    ActiveChats = property(_GetActiveChats,
    doc="""Queries a list of active chats.

    :type: `ChatCollection`
    """)

    def _GetActiveFileTransfers(self):
        return FileTransferCollection(self, self._Search('ACTIVEFILETRANSFERS'))

    ActiveFileTransfers = property(_GetActiveFileTransfers,
    doc="""Queries currently active file transfers.

    :type: `FileTransferCollection`
    """)

    def _GetApiWrapperVersion(self):
        import pkg_resources
        return pkg_resources.get_distribution("Skype4Py").version

    ApiWrapperVersion = property(_GetApiWrapperVersion,
    doc="""Returns Skype4Py version.

    :type: str
    """)

    def _GetAttachmentStatus(self):
        return self._Api.attachment_status

    AttachmentStatus = property(_GetAttachmentStatus,
    doc="""Queries the attachment status of the Skype client.

    :type: `enums`.apiAttach*
    """)

    def _GetBookmarkedChats(self):
        return ChatCollection(self, self._Search('BOOKMARKEDCHATS'))

    BookmarkedChats = property(_GetBookmarkedChats,
    doc="""Queries a list of bookmarked chats.

    :type: `ChatCollection`
    """)

    def _GetCache(self):
        return self._Cache

    def _SetCache(self, Value):
        self._Cache = bool(Value)

    Cache = property(_GetCache, _SetCache,
    doc="""Queries/sets the status of internal cache. The internal API cache is used
    to cache Skype object properties and global parameters.

    :type: bool
    """)

    def _GetChats(self):
        return ChatCollection(self, self._Search('CHATS'))

    Chats = property(_GetChats,
    doc="""Queries a list of chats.

    :type: `ChatCollection`
    """)

    def _GetClient(self):
        return self._Client

    Client = property(_GetClient,
    doc="""Queries the user interface control object.

    :type: `Client`
    """)

    def _GetCommandId(self):
        return True

    def _SetCommandId(self, Value):
        if not Value:
            raise SkypeError(0, 'CommandId may not be False')

    CommandId = property(_GetCommandId, _SetCommandId,
    doc="""Queries/sets the status of automatic command identifiers.

    :type: bool

    :note: Currently the only supported value is True.
    """)

    def _GetConferences(self):
        cids = []
        for c in self.Calls():
            cid = c.ConferenceId
            if cid > 0 and cid not in cids:
                cids.append(cid)
        return ConferenceCollection(self, cids)

    Conferences = property(_GetConferences,
    doc="""Queries a list of call conferences.

    :type: `ConferenceCollection`
    """)

    def _GetConnectionStatus(self):
        return str(self.Variable('CONNSTATUS'))

    ConnectionStatus = property(_GetConnectionStatus,
    doc="""Queries the connection status of the Skype client.

    :type: `enums`.con*
    """)

    def _GetConvert(self):
        return self._Convert

    Convert = property(_GetConvert,
    doc="""Queries the conversion object.

    :type: `Conversion`
    """)

    def _GetCurrentUser(self):
        return User(self, self.CurrentUserHandle)

    CurrentUser = property(_GetCurrentUser,
    doc="""Queries the current user object.

    :type: `user.User`
    """)

    def _GetCurrentUserHandle(self):
        return str(self.Variable('CURRENTUSERHANDLE'))

    CurrentUserHandle = property(_GetCurrentUserHandle,
    doc="""Queries the Skypename of the current user.

    :type: str
    """)

    def _GetCurrentUserProfile(self):
        return self._Profile

    CurrentUserProfile = property(_GetCurrentUserProfile,
    doc="""Queries the user profile object.

    :type: `Profile`
    """)

    def _GetCurrentUserStatus(self):
        return str(self.Variable('USERSTATUS'))

    def _SetCurrentUserStatus(self, Value):
        self.Variable('USERSTATUS', str(Value))

    CurrentUserStatus = property(_GetCurrentUserStatus, _SetCurrentUserStatus,
    doc="""Queries/sets the online status of the current user.

    :type: `enums`.ols*
    """)

    def _GetCustomGroups(self):
        return GroupCollection(self, self._Search('GROUPS', 'CUSTOM'))

    CustomGroups = property(_GetCustomGroups,
    doc="""Queries the list of custom contact groups. Custom groups are contact groups defined by the user.

    :type: `GroupCollection`
    """)

    def _GetFileTransfers(self):
        return FileTransferCollection(self, self._Search('FILETRANSFERS'))

    FileTransfers = property(_GetFileTransfers,
    doc="""Queries all file transfers.

    :type: `FileTransferCollection`
    """)

    def _GetFocusedContacts(self):
        # we have to use _DoCommand() directly because for unknown reason the API returns
        # "CONTACTS FOCUSED" instead of "CONTACTS_FOCUSED" (note the space instead of "_")
        return UserCollection(self, split(chop(self._DoCommand('GET CONTACTS_FOCUSED', 'CONTACTS FOCUSED'), 2)[-1]))

    FocusedContacts = property(_GetFocusedContacts,
    doc="""Queries a list of contacts selected in the contacts list.

    :type: `UserCollection`
    """)

    def _GetFriendlyName(self):
        return self._Api.friendly_name

    def _SetFriendlyName(self, Value):
        self._Api.set_friendly_name(tounicode(Value))

    FriendlyName = property(_GetFriendlyName, _SetFriendlyName,
    doc="""Queries/sets a "friendly" name for an application.

    :type: unicode
    """)

    def _GetFriends(self):
        return UserCollection(self, self._Search('FRIENDS'))

    Friends = property(_GetFriends,
    doc="""Queries the users in a contact list.

    :type: `UserCollection`
    """)

    def _GetGroups(self):
        return GroupCollection(self, self._Search('GROUPS', 'ALL'))

    Groups = property(_GetGroups,
    doc="""Queries the list of all contact groups.

    :type: `GroupCollection`
    """)

    def _GetHardwiredGroups(self):
        return GroupCollection(self, self._Search('GROUPS', 'HARDWIRED'))

    HardwiredGroups = property(_GetHardwiredGroups,
    doc="""Queries the list of hardwired contact groups. Hardwired groups are "smart" contact groups,
    defined by Skype, that cannot be removed.

    :type: `GroupCollection`
    """)

    def _GetMissedCalls(self):
        return CallCollection(self, self._Search('MISSEDCALLS'))

    MissedCalls = property(_GetMissedCalls,
    doc="""Queries a list of missed calls.

    :type: `CallCollection`
    """)

    def _GetMissedChats(self):
        return ChatCollection(self, self._Search('MISSEDCHATS'))

    MissedChats = property(_GetMissedChats,
    doc="""Queries a list of missed chats.

    :type: `ChatCollection`
    """)

    def _GetMissedMessages(self):
        return ChatMessageCollection(self, self._Search('MISSEDCHATMESSAGES'))

    MissedMessages = property(_GetMissedMessages,
    doc="""Queries a list of missed chat messages.

    :type: `ChatMessageCollection`
    """)

    def _GetMissedSmss(self):
        return SmsMessageCollection(self, self._Search('MISSEDSMSS'))

    MissedSmss = property(_GetMissedSmss,
    doc="""Requests a list of all missed SMS messages.

    :type: `SmsMessageCollection`
    """)

    def _GetMissedVoicemails(self):
        return VoicemailCollection(self, self._Search('MISSEDVOICEMAILS'))

    MissedVoicemails = property(_GetMissedVoicemails,
    doc="""Requests a list of missed voicemails.

    :type: `VoicemailCollection`
    """)

    def _GetMute(self):
        return self.Variable('MUTE') == 'ON'

    def _SetMute(self, Value):
        self.Variable('MUTE', cndexp(Value, 'ON', 'OFF'))

    Mute = property(_GetMute, _SetMute,
    doc="""Queries/sets the mute status of the Skype client.

    Type: bool
    Note: This value can be set only when there is an active call.

    :type: bool
    """)

    def _GetPredictiveDialerCountry(self):
        return str(self.Variable('PREDICTIVE_DIALER_COUNTRY'))

    PredictiveDialerCountry = property(_GetPredictiveDialerCountry,
    doc="""Returns predictive dialler country as an ISO code.

    :type: str
    """)

    def _GetProtocol(self):
        return self._Api.protocol

    def _SetProtocol(self, Value):
        self._DoCommand('PROTOCOL %s' % Value)
        self._Api.protocol = int(Value)

    Protocol = property(_GetProtocol, _SetProtocol,
    doc="""Queries/sets the protocol version used by the Skype client.

    :type: int
    """)

    def _GetRecentChats(self):
        return ChatCollection(self, self._Search('RECENTCHATS'))

    RecentChats = property(_GetRecentChats,
    doc="""Queries a list of recent chats.

    :type: `ChatCollection`
    """)

    def _GetSettings(self):
        return self._Settings

    Settings = property(_GetSettings,
    doc="""Queries the settings for Skype general parameters.

    :type: `Settings`
    """)

    def _GetSilentMode(self):
        return self._Property('SILENT_MODE', '', '', Cache=False) == 'ON'

    def _SetSilentMode(self, Value):
        self._Property('SILENT_MODE', '', '', cndexp(Value, 'ON', 'OFF'), Cache=False)

    SilentMode = property(_GetSilentMode, _SetSilentMode,
    doc="""Returns/sets Skype silent mode status.

    :type: bool
    """)

    def _GetSmss(self):
        return SmsMessageCollection(self, self._Search('SMSS'))

    Smss = property(_GetSmss,
    doc="""Requests a list of all SMS messages.

    :type: `SmsMessageCollection`
    """)

    def _GetTimeout(self):
        return self._Timeout

    def _SetTimeout(self, Value):
        if not isinstance(Value, (int, long, float)):
            raise TypeError('%s: wrong type, expected float (seconds), int or long (milliseconds)' %
                repr(type(Value)))
        self._Timeout = Value

    Timeout = property(_GetTimeout, _SetTimeout,
    doc="""Queries/sets the wait timeout value. This timeout value applies to every command sent
    to the Skype API and to attachment requests (see `Attach`). If a response is not received
    during the timeout period, an `SkypeAPIError` exception is raised.

    The units depend on the type. For float it is the number of seconds (or fractions thereof),
    for int or long it is the number of milliseconds. Floats are commonly used in Python modules
    to express timeouts (time.sleep() for example). Milliseconds are supported because that's
    what the Skype4COM library uses. Skype4Py support for real float timeouts was introduced
    in version 1.0.31.1.

    The default value is 30000 milliseconds (int).

    :type: float, int or long
    """)

    def _GetUsersWaitingAuthorization(self):
        return UserCollection(self, self._Search('USERSWAITINGMYAUTHORIZATION'))

    UsersWaitingAuthorization = property(_GetUsersWaitingAuthorization,
    doc="""Queries the list of users waiting for authorization.

    :type: `UserCollection`
    """)

    def _GetVersion(self):
        return str(self.Variable('SKYPEVERSION'))

    Version = property(_GetVersion,
    doc="""Queries the application version of the Skype client.

    :type: str
    """)

    def _GetVoicemails(self):
        return VoicemailCollection(self, self._Search('VOICEMAILS'))

    Voicemails = property(_GetVoicemails,
    doc="""Queries a list of voicemails.

    :type: `VoicemailCollection`
    """)


class SkypeEvents(object):
    """Events defined in `Skype`.

    See `EventHandlingBase` for more information on events.
    """

    def ApplicationConnecting(self, App, Users):
        """This event is triggered when list of users connecting to an application changes.

        :Parameters:
          App : `Application`
            Application object.
          Users : `UserCollection`
            Connecting users.
        """

    def ApplicationDatagram(self, App, Stream, Text):
        """This event is caused by the arrival of an application datagram.

        :Parameters:
          App : `Application`
            Application object.
          Stream : `ApplicationStream`
            Application stream that received the datagram.
          Text : unicode
            The datagram text.
        """

    def ApplicationReceiving(self, App, Streams):
        """This event is triggered when list of application receiving streams changes.

        :Parameters:
          App : `Application`
            Application object.
          Streams : `ApplicationStreamCollection`
            Application receiving streams.
        """

    def ApplicationSending(self, App, Streams):
        """This event is triggered when list of application sending streams changes.

        :Parameters:
          App : `Application`
            Application object.
          Streams : `ApplicationStreamCollection`
            Application sending streams.
        """

    def ApplicationStreams(self, App, Streams):
        """This event is triggered when list of application streams changes.

        :Parameters:
          App : `Application`
            Application object.
          Streams : `ApplicationStreamCollection`
            Application streams.
        """

    def AsyncSearchUsersFinished(self, Cookie, Users):
        """This event occurs when an asynchronous search is completed.

        :Parameters:
          Cookie : int
            Search identifier as returned by `Skype.AsyncSearchUsers`.
          Users : `UserCollection`
            Found users.

        :see: `Skype.AsyncSearchUsers`
        """

    def AttachmentStatus(self, Status):
        """This event is caused by a change in the status of an attachment to the Skype API.

        :Parameters:
          Status : `enums`.apiAttach*
            New attachment status.
        """

    def AutoAway(self, Automatic):
        """This event is caused by a change of auto away status.

        :Parameters:
          Automatic : bool
            New auto away status.
        """

    def CallDtmfReceived(self, Call, Code):
        """This event is caused by a call DTMF event.

        :Parameters:
          Call : `Call`
            Call object.
          Code : str
            Received DTMF code.
        """

    def CallHistory(self):
        """This event is caused by a change in call history.
        """

    def CallInputStatusChanged(self, Call, Active):
        """This event is caused by a change in the Call voice input status change.

        :Parameters:
          Call : `Call`
            Call object.
          Active : bool
            New voice input status (active when True).
        """

    def CallSeenStatusChanged(self, Call, Seen):
        """This event occurs when the seen status of a call changes.

        :Parameters:
          Call : `Call`
            Call object.
          Seen : bool
            True if call was seen.

        :see: `Call.Seen`
        """

    def CallStatus(self, Call, Status):
        """This event is caused by a change in call status.

        :Parameters:
          Call : `Call`
            Call object.
          Status : `enums`.cls*
            New status of the call.
        """

    def CallTransferStatusChanged(self, Call, Status):
        """This event occurs when a call transfer status changes.

        :Parameters:
          Call : `Call`
            Call object.
          Status : `enums`.cls*
            New status of the call transfer.
        """

    def CallVideoReceiveStatusChanged(self, Call, Status):
        """This event occurs when a call video receive status changes.

        :Parameters:
          Call : `Call`
            Call object.
          Status : `enums`.vss*
            New video receive status of the call.
        """

    def CallVideoSendStatusChanged(self, Call, Status):
        """This event occurs when a call video send status changes.

        :Parameters:
          Call : `Call`
            Call object.
          Status : `enums`.vss*
            New video send status of the call.
        """

    def CallVideoStatusChanged(self, Call, Status):
        """This event occurs when a call video status changes.

        :Parameters:
          Call : `Call`
            Call object.
          Status : `enums`.cvs*
            New video status of the call.
        """

    def ChatMemberRoleChanged(self, Member, Role):
        """This event occurs when a chat member role changes.

        :Parameters:
          Member : `ChatMember`
            Chat member object.
          Role : `enums`.chatMemberRole*
            New member role.
        """

    def ChatMembersChanged(self, Chat, Members):
        """This event occurs when a list of chat members change.

        :Parameters:
          Chat : `Chat`
            Chat object.
          Members : `UserCollection`
            Chat members.
        """

    def ChatWindowState(self, Chat, State):
        """This event occurs when chat window is opened or closed.

        :Parameters:
          Chat : `Chat`
            Chat object.
          State : bool
            True if the window was opened or False if closed.
        """

    def ClientWindowState(self, State):
        """This event occurs when the state of the client window changes.

        :Parameters:
          State : `enums`.wnd*
            New window state.
        """

    def Command(self, command):
        """This event is triggered when a command is sent to the Skype API.

        :Parameters:
          command : `Command`
            Command object.
        """

    def ConnectionStatus(self, Status):
        """This event is caused by a connection status change.

        :Parameters:
          Status : `enums`.con*
            New connection status.
        """

    def ContactsFocused(self, Username):
        """This event is caused by a change in contacts focus.

        :Parameters:
          Username : str
            Name of the user that was focused or empty string if focus was lost.
        """

    def Error(self, command, Number, Description):
        """This event is triggered when an error occurs during execution of an API command.

        :Parameters:
          command : `Command`
            Command object that caused the error.
          Number : int
            Error number returned by the Skype API.
          Description : unicode
            Description of the error.
        """

    def FileTransferStatusChanged(self, Transfer, Status):
        """This event occurs when a file transfer status changes.

        :Parameters:
          Transfer : `FileTransfer`
            File transfer object.
          Status : `enums`.fileTransferStatus*
            New status of the file transfer.
        """

    def GroupDeleted(self, GroupId):
        """This event is caused by a user deleting a custom contact group.

        :Parameters:
          GroupId : int
            Id of the deleted group.
        """

    def GroupExpanded(self, Group, Expanded):
        """This event is caused by a user expanding or collapsing a group in the contacts tab.

        :Parameters:
          Group : `Group`
            Group object.
          Expanded : bool
            Tells if the group is expanded (True) or collapsed (False).
        """

    def GroupUsers(self, Group, Count):
        """This event is caused by a change in a contact group members.

        :Parameters:
          Group : `Group`
            Group object.
          Count : int
            Number of group members.

        :note: This event is different from its Skype4COM equivalent in that the second
               parameter is number of users instead of `UserCollection` object. This
               object may be obtained using ``Group.Users`` property.
        """

    def GroupVisible(self, Group, Visible):
        """This event is caused by a user hiding/showing a group in the contacts tab.

        :Parameters:
          Group : `Group`
            Group object.
          Visible : bool
            Tells if the group is visible or not.
        """

    def MessageHistory(self, Username):
        """This event is caused by a change in message history.

        :Parameters:
          Username : str
            Name of the user whose message history changed.
        """

    def MessageStatus(self, Message, Status):
        """This event is caused by a change in chat message status.

        :Parameters:
          Message : `ChatMessage`
            Chat message object.
          Status : `enums`.cms*
            New status of the chat message.
        """

    def Mute(self, Mute):
        """This event is caused by a change in mute status.

        :Parameters:
          Mute : bool
            New mute status.
        """

    def Notify(self, Notification):
        """This event is triggered whenever Skype client sends a notification.

        :Parameters:
          Notification : unicode
            Notification string.

        :note: Use this event only if there is no dedicated one.
        """

    def OnlineStatus(self, User, Status):
        """This event is caused by a change in the online status of a user.

        :Parameters:
          User : `User`
            User object.
          Status : `enums`.ols*
            New online status of the user.
        """

    def PluginEventClicked(self, Event):
        """This event occurs when a user clicks on a plug-in event.

        :Parameters:
          Event : `PluginEvent`
            Plugin event object.
        """

    def PluginMenuItemClicked(self, MenuItem, Users, PluginContext, ContextId):
        """This event occurs when a user clicks on a plug-in menu item.

        :Parameters:
          MenuItem : `PluginMenuItem`
            Menu item object.
          Users : `UserCollection`
            Users this item refers to.
          PluginContext : unicode
            Plug-in context.
          ContextId : str or int
            Context Id. Chat name for chat context or Call ID for call context.

        :see: `PluginMenuItem`
        """

    def Reply(self, command):
        """This event is triggered when the API replies to a command object.

        :Parameters:
          command : `Command`
            Command object.
        """

    def SilentModeStatusChanged(self, Silent):
        """This event occurs when a silent mode is switched off.

        :Parameters:
          Silent : bool
            Skype client silent status.
        """

    def SmsMessageStatusChanged(self, Message, Status):
        """This event is caused by a change in the SMS message status.

        :Parameters:
          Message : `SmsMessage`
            SMS message object.
          Status : `enums`.smsMessageStatus*
            New status of the SMS message.
        """

    def SmsTargetStatusChanged(self, Target, Status):
        """This event is caused by a change in the SMS target status.

        :Parameters:
          Target : `SmsTarget`
            SMS target object.
          Status : `enums`.smsTargetStatus*
            New status of the SMS target.
        """

    def UserAuthorizationRequestReceived(self, User):
        """This event occurs when user sends you an authorization request.

        :Parameters:
          User : `User`
            User object.
        """

    def UserMood(self, User, MoodText):
        """This event is caused by a change in the mood text of the user.

        :Parameters:
          User : `User`
            User object.
          MoodText : unicode
            New mood text.
        """

    def UserStatus(self, Status):
        """This event is caused by a user status change.

        :Parameters:
          Status : `enums`.cus*
            New user status.
        """

    def VoicemailStatus(self, Mail, Status):
        """This event is caused by a change in voicemail status.

        :Parameters:
          Mail : `Voicemail`
            Voicemail object.
          Status : `enums`.vms*
            New status of the voicemail.
        """

    def WallpaperChanged(self, Path):
        """This event occurs when client wallpaper changes.

        :Parameters:
          Path : str
            Path to new wallpaper bitmap.
        """


Skype._AddEvents(SkypeEvents)



apiAttachAvailable = u'API\u53ef\u7528'
apiAttachNotAvailable = u'\u7121\u6cd5\u4f7f\u7528'
apiAttachPendingAuthorization = u'\u7b49\u5f85\u6388\u6b0a'
apiAttachRefused = u'\u88ab\u62d2'
apiAttachSuccess = u'\u6210\u529f'
apiAttachUnknown = u'\u4e0d\u660e'
budDeletedFriend = u'\u5df2\u5f9e\u670b\u53cb\u540d\u55ae\u522a\u9664'
budFriend = u'\u670b\u53cb'
budNeverBeenFriend = u'\u5f9e\u672a\u5217\u5165\u670b\u53cb\u540d\u55ae'
budPendingAuthorization = u'\u7b49\u5f85\u6388\u6b0a'
budUnknown = u'\u4e0d\u660e'
cfrBlockedByRecipient = u'\u901a\u8a71\u88ab\u63a5\u6536\u65b9\u5c01\u9396'
cfrMiscError = u'\u5176\u4ed6\u932f\u8aa4'
cfrNoCommonCodec = u'\u6c92\u6709\u901a\u7528 codec'
cfrNoProxyFound = u'\u627e\u4e0d\u5230 proxy'
cfrNotAuthorizedByRecipient = u'\u76ee\u524d\u4f7f\u7528\u8005\u672a\u7372\u63a5\u53d7\u65b9\u6388\u6b0a'
cfrRecipientNotFriend = u'\u63a5\u6536\u65b9\u4e0d\u662f\u670b\u53cb'
cfrRemoteDeviceError = u'\u9060\u7aef\u97f3\u6548\u88dd\u7f6e\u554f\u984c'
cfrSessionTerminated = u'\u901a\u8a71\u7d50\u675f'
cfrSoundIOError = u'\u97f3\u6548\u8f38\u5165/\u8f38\u51fa\u932f\u8aa4'
cfrSoundRecordingError = u'\u9304\u97f3\u932f\u8aa4'
cfrUnknown = u'\u4e0d\u660e'
cfrUserDoesNotExist = u'\u4f7f\u7528\u8005/\u96fb\u8a71\u865f\u78bc\u4e0d\u5b58\u5728'
cfrUserIsOffline = u'\u4ed6/\u5979\u4e0d\u5728\u7dda\u4e0a'
chsAllCalls = u'\u820a\u7248\u5c0d\u8a71'
chsDialog = u'\u5c0d\u8a71'
chsIncomingCalls = u'\u9700\u8981\u591a\u65b9\u63a5\u53d7'
chsLegacyDialog = u'\u820a\u7248\u5c0d\u8a71'
chsMissedCalls = u'\u5c0d\u8a71'
chsMultiNeedAccept = u'\u9700\u8981\u591a\u65b9\u63a5\u53d7'
chsMultiSubscribed = u'\u591a\u65b9\u8a02\u7528'
chsOutgoingCalls = u'\u591a\u65b9\u8a02\u7528'
chsUnknown = u'\u4e0d\u660e'
chsUnsubscribed = u'\u53d6\u6d88\u8a02\u7528'
clsBusy = u'\u5fd9\u7dda'
clsCancelled = u'\u5df2\u53d6\u6d88'
clsEarlyMedia = u'\u9810\u524d\u5a92\u9ad4 (Early Media) \u64ad\u653e\u4e2d'
clsFailed = u'\u5f88\u62b1\u6b49,\u64a5\u865f\u5931\u6557'
clsFinished = u'\u5df2\u7d50\u675f'
clsInProgress = u'\u901a\u8a71\u4e2d'
clsLocalHold = u'\u672c\u6a5f\u901a\u8a71\u4fdd\u7559\u4e2d'
clsMissed = u'\u672a\u63a5\u96fb\u8a71'
clsOnHold = u'\u7b49\u5f85\u4e2d'
clsRefused = u'\u88ab\u62d2'
clsRemoteHold = u'\u9060\u7aef\u901a\u8a71\u4fdd\u7559\u4e2d'
clsRinging = u'\u6b63\u5728\u64a5\u6253'
clsRouting = u'\u9023\u63a5\u4e2d'
clsTransferred = u'\u4e0d\u660e'
clsTransferring = u'\u4e0d\u660e'
clsUnknown = u'\u4e0d\u660e'
clsUnplaced = u'\u5f9e\u672a\u64a5\u6253'
clsVoicemailBufferingGreeting = u'\u554f\u5019\u8a9e\u7de9\u885d\u8655\u7406\u4e2d'
clsVoicemailCancelled = u'\u5df2\u53d6\u6d88\u8a9e\u97f3\u90f5\u4ef6'
clsVoicemailFailed = u'\u7559\u8a00\u4e0a\u50b3\u5931\u6557'
clsVoicemailPlayingGreeting = u'\u554f\u5019\u8a9e\u64ad\u653e\u4e2d'
clsVoicemailRecording = u'\u9304\u88fd\u7559\u8a00'
clsVoicemailSent = u'\u5df2\u50b3\u9001\u8a9e\u97f3\u90f5\u4ef6'
clsVoicemailUploading = u'\u6b63\u5728\u4e0a\u8f09\u8a9e\u97f3\u90f5\u4ef6'
cltIncomingP2P = u'\u64a5\u5165\u540c\u5115 (P2P) \u96fb\u8a71'
cltIncomingPSTN = u'\u64a5\u5165\u96fb\u8a71'
cltOutgoingP2P = u'\u64a5\u51fa\u540c\u5115 (P2P) \u96fb\u8a71'
cltOutgoingPSTN = u'\u64a5\u51fa\u96fb\u8a71'
cltUnknown = u'\u4e0d\u660e'
cmeAddedMembers = u'\u589e\u52a0\u6210\u54e1'
cmeCreatedChatWith = u'\u5df2\u50b3\u9001\u5373\u6642\u8a0a\u606f'
cmeEmoted = u'\u4e0d\u660e'
cmeLeft = u'\u5df2\u96e2\u958b'
cmeSaid = u'\u5df2\u8aaa\u904e'
cmeSawMembers = u'\u770b\u5230\u6210\u54e1'
cmeSetTopic = u'\u8a2d\u7acb\u8a71\u984c'
cmeUnknown = u'\u4e0d\u660e'
cmsRead = u'\u5df2\u95b1\u8b80'
cmsReceived = u'\u5df2\u63a5\u6536'
cmsSending = u'\u50b3\u9001\u4e2d...'
cmsSent = u'\u5df2\u50b3\u9001'
cmsUnknown = u'\u4e0d\u660e'
conConnecting = u'\u9023\u7dda\u4e2d'
conOffline = u'\u96e2\u7dda'
conOnline = u'\u4e0a\u7dda\u4e2d'
conPausing = u'\u6b63\u5728\u66ab\u505c'
conUnknown = u'\u4e0d\u660e'
cusAway = u'\u66ab\u6642\u96e2\u958b'
cusDoNotDisturb = u'\u8acb\u52ff\u6253\u64fe'
cusInvisible = u'\u96b1\u85cf'
cusLoggedOut = u'\u96e2\u7dda'
cusNotAvailable = u'\u7121\u6cd5\u4f7f\u7528'
cusOffline = u'\u96e2\u7dda'
cusOnline = u'\u4e0a\u7dda\u4e2d'
cusSkypeMe = u'\u958b\u653e\u804a\u5929'
cusUnknown = u'\u4e0d\u660e'
cvsBothEnabled = u'\u8996\u8a0a\u50b3\u9001\u8207\u63a5\u6536'
cvsNone = u'\u7121\u8996\u8a0a'
cvsReceiveEnabled = u'\u8996\u8a0a\u63a5\u6536'
cvsSendEnabled = u'\u8996\u8a0a\u50b3\u9001'
cvsUnknown = u''
grpAllFriends = u'\u6240\u6709\u670b\u53cb'
grpAllUsers = u'\u6240\u6709\u4f7f\u7528\u8005'
grpCustomGroup = u'\u81ea\u8a02'
grpOnlineFriends = u'\u7dda\u4e0a\u670b\u53cb'
grpPendingAuthorizationFriends = u'\u7b49\u5f85\u6388\u6b0a'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'\u6700\u8fd1\u806f\u7d61\u904e\u7684\u4f7f\u7528\u8005'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Skype \u670b\u53cb'
grpSkypeOutFriends = u'SkypeOut \u670b\u53cb'
grpUngroupedFriends = u'\u672a\u5206\u7d44\u7684\u670b\u53cb'
grpUnknown = u'\u4e0d\u660e'
grpUsersAuthorizedByMe = u'\u7d93\u6211\u6388\u6b0a'
grpUsersBlockedByMe = u'\u7d93\u6211\u5c01\u9396'
grpUsersWaitingMyAuthorization = u'\u7b49\u5f85\u6211\u7684\u6388\u6b0a'
leaAddDeclined = u'\u52a0\u5165\u906d\u62d2'
leaAddedNotAuthorized = u'\u88ab\u52a0\u5165\u8005\u5fc5\u9808\u7372\u5f97\u6388\u6b0a'
leaAdderNotFriend = u'\u52a0\u5165\u8005\u5fc5\u9808\u662f\u670b\u53cb'
leaUnknown = u'\u4e0d\u660e'
leaUnsubscribe = u'\u53d6\u6d88\u8a02\u7528'
leaUserIncapable = u'\u4f7f\u7528\u8005\u7121\u6cd5\u901a\u8a71'
leaUserNotFound = u'\u627e\u4e0d\u5230\u4f7f\u7528\u8005'
olsAway = u'\u66ab\u6642\u96e2\u958b'
olsDoNotDisturb = u'\u8acb\u52ff\u6253\u64fe'
olsNotAvailable = u'\u7121\u6cd5\u4f7f\u7528'
olsOffline = u'\u96e2\u7dda'
olsOnline = u'\u4e0a\u7dda\u4e2d'
olsSkypeMe = u'\u958b\u653e\u804a\u5929'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'\u4e0d\u660e'
smsMessageStatusComposing = u''
smsMessageStatusDelivered = u''
smsMessageStatusFailed = u''
smsMessageStatusRead = u''
smsMessageStatusReceived = u''
smsMessageStatusSendingToServer = u''
smsMessageStatusSentToServer = u''
smsMessageStatusSomeTargetsFailed = u''
smsMessageStatusUnknown = u''
smsMessageTypeCCRequest = u''
smsMessageTypeCCSubmit = u''
smsMessageTypeIncoming = u''
smsMessageTypeOutgoing = u''
smsMessageTypeUnknown = u''
smsTargetStatusAcceptable = u''
smsTargetStatusAnalyzing = u''
smsTargetStatusDeliveryFailed = u''
smsTargetStatusDeliveryPending = u''
smsTargetStatusDeliverySuccessful = u''
smsTargetStatusNotRoutable = u''
smsTargetStatusUndefined = u''
smsTargetStatusUnknown = u''
usexFemale = u'\u5973'
usexMale = u'\u7537'
usexUnknown = u'\u4e0d\u660e'
vmrConnectError = u'\u9023\u63a5\u932f\u8aa4'
vmrFileReadError = u'\u6a94\u6848\u8b80\u53d6\u932f\u8aa4'
vmrFileWriteError = u'\u6a94\u6848\u5beb\u5165\u932f\u8aa4'
vmrMiscError = u'\u5176\u4ed6\u932f\u8aa4'
vmrNoError = u'\u7121\u932f\u8aa4'
vmrNoPrivilege = u'\u7121\u8a9e\u97f3\u90f5\u4ef6\u6b0a\u9650'
vmrNoVoicemail = u'\u6c92\u6709\u9019\u7a2e\u8a9e\u97f3\u90f5\u4ef6'
vmrPlaybackError = u'\u64ad\u653e\u932f\u8aa4'
vmrRecordingError = u'\u9304\u97f3\u932f\u8aa4'
vmrUnknown = u'\u4e0d\u660e'
vmsBlank = u'\u7a7a\u767d'
vmsBuffering = u'\u7de9\u885d\u8655\u7406\u4e2d'
vmsDeleting = u'\u6b63\u5728\u522a\u9664'
vmsDownloading = u'\u6b63\u5728\u4e0b\u8f09'
vmsFailed = u'\u5931\u6557'
vmsNotDownloaded = u'\u672a\u4e0b\u8f09'
vmsPlayed = u'\u5df2\u64ad\u653e'
vmsPlaying = u'\u6b63\u5728\u64ad\u653e'
vmsRecorded = u'\u5df2\u9304\u597d'
vmsRecording = u'\u9304\u88fd\u7559\u8a00'
vmsUnknown = u'\u4e0d\u660e'
vmsUnplayed = u'\u672a\u64ad\u653e'
vmsUploaded = u'\u5df2\u4e0a\u8f09'
vmsUploading = u'\u6b63\u5728\u4e0a\u8f09'
vmtCustomGreeting = u'\u81ea\u8a02\u554f\u5019\u8a9e'
vmtDefaultGreeting = u'\u9810\u8a2d\u554f\u5019\u8a9e'
vmtIncoming = u'\u65b0\u7559\u8a00'
vmtOutgoing = u'\u64a5\u51fa'
vmtUnknown = u'\u4e0d\u660e'
vssAvailable = u'\u53ef\u7528'
vssNotAvailable = u'\u7121\u6cd5\u4f7f\u7528'
vssPaused = u'\u5df2\u66ab\u505c'
vssRejected = u'\u5df2\u88ab\u62d2\u7d55'
vssRunning = u'\u6b63\u5728\u9032\u884c'
vssStarting = u'\u958b\u59cb'
vssStopping = u'\u6b63\u5728\u505c\u6b62'
vssUnknown = u'\u4e0d\u660e'



apiAttachAvailable = u'API je k dispozici'
apiAttachNotAvailable = u'Nedostupn\xfd'
apiAttachPendingAuthorization = u'Nevyr\xedzen\xe1 autorizace'
apiAttachRefused = u'Odm\xedtnuto'
apiAttachSuccess = u'\xdaspech'
apiAttachUnknown = u'Nezn\xe1m\xfd'
budDeletedFriend = u'Odstranen ze seznamu pr\xe1tel'
budFriend = u'Pr\xedtel'
budNeverBeenFriend = u'Nikdy nebyl v seznamu pr\xe1tel'
budPendingAuthorization = u'Nevyr\xedzen\xe1 autorizace'
budUnknown = u'Nezn\xe1m\xfd'
cfrBlockedByRecipient = u'Hovor je blokov\xe1n pr\xedjemcem.'
cfrMiscError = u'Jin\xe1 chyba'
cfrNoCommonCodec = u'Neobvykl\xfd kodek'
cfrNoProxyFound = u'Server proxy nebyl nalezen.'
cfrNotAuthorizedByRecipient = u'Aktu\xe1ln\xed u\u017eivatel nen\xed pr\xedjemcem autorizov\xe1n.'
cfrRecipientNotFriend = u'Pr\xedjemce nen\xed pr\xedtel.'
cfrRemoteDeviceError = u'Chyba zvukov\xe9ho zar\xedzen\xed volan\xe9ho'
cfrSessionTerminated = u'Hovor ukoncen'
cfrSoundIOError = u'Chyba zvukov\xe9ho V/V'
cfrSoundRecordingError = u'Chyba nahr\xe1v\xe1n\xed zvuku'
cfrUnknown = u'Nezn\xe1m\xfd'
cfrUserDoesNotExist = u'U\u017eivatel/telefonn\xed c\xedslo neexistuje.'
cfrUserIsOffline = u'On nebo Ona je Offline'
chsAllCalls = u'Dialog ve star\xe9m stylu'
chsDialog = u'Dialog'
chsIncomingCalls = u'S v\xedce \xfacastn\xedky, je treba prijet\xed'
chsLegacyDialog = u'Dialog ve star\xe9m stylu'
chsMissedCalls = u'Dialog'
chsMultiNeedAccept = u'S v\xedce \xfacastn\xedky, je treba prijet\xed'
chsMultiSubscribed = u'S v\xedce \xfacastn\xedky'
chsOutgoingCalls = u'S v\xedce \xfacastn\xedky'
chsUnknown = u'Nezn\xe1m\xfd'
chsUnsubscribed = u'Odebran\xfd ze seznamu'
clsBusy = u'Obsazeno'
clsCancelled = u'Zru\u0161eno'
clsEarlyMedia = u'Prehr\xe1v\xe1n\xed m\xe9di\xed pred prijet\xedm hovoru'
clsFailed = u'Bohu\u017eel, ne\xfaspe\u0161n\xe9 vol\xe1n\xed!'
clsFinished = u'Ukonceno'
clsInProgress = u'Prob\xedh\xe1 hovor'
clsLocalHold = u'Pridr\u017eeno m\xedstne'
clsMissed = u'Zme\u0161kan\xfd hovor'
clsOnHold = u'Pridr\u017een'
clsRefused = u'Odm\xedtnuto'
clsRemoteHold = u'Pridr\u017eeno vzd\xe1len\xfdm u\u017eivatelem'
clsRinging = u'vol\xe1te'
clsRouting = u'Smerov\xe1n\xed'
clsTransferred = u'Nezn\xe1m\xfd'
clsTransferring = u'Nezn\xe1m\xfd'
clsUnknown = u'Nezn\xe1m\xfd'
clsUnplaced = u'Nekonal se'
clsVoicemailBufferingGreeting = u'Ukl\xe1d\xe1n\xed pozdravu do vyrovn\xe1vac\xed pameti'
clsVoicemailCancelled = u'Hlasov\xe1 zpr\xe1va byla zru\u0161ena'
clsVoicemailFailed = u'Hlasov\xe1 zpr\xe1va ne\xfaspe\u0161n\xe1'
clsVoicemailPlayingGreeting = u'Prehr\xe1v\xe1n\xed pozdravu'
clsVoicemailRecording = u'Nahr\xe1v\xe1n\xed hlasov\xe9 zpr\xe1vy'
clsVoicemailSent = u'Hlasov\xe1 zpr\xe1va byla odesl\xe1na'
clsVoicemailUploading = u'Odes\xedl\xe1n\xed hlasov\xe9 zpr\xe1vy'
cltIncomingP2P = u'Pr\xedchoz\xed hovor v s\xedti P2P'
cltIncomingPSTN = u'Pr\xedchoz\xed telefonn\xed hovor'
cltOutgoingP2P = u'Odchoz\xed hovor v s\xedti P2P'
cltOutgoingPSTN = u'Odchoz\xed telefonn\xed hovor'
cltUnknown = u'Nezn\xe1m\xfd'
cmeAddedMembers = u'Byli prizv\xe1ni clenov\xe9.'
cmeCreatedChatWith = u'Byl vytvoren chat s v\xedce \xfacastn\xedky.'
cmeEmoted = u'Nezn\xe1m\xfd'
cmeLeft = u'Nekdo opustil chat nebo nebyl pribr\xe1n.'
cmeSaid = u'Rekl(a)'
cmeSawMembers = u'\xdacastn\xedk chatu videl ostatn\xed.'
cmeSetTopic = u'Zmena t\xe9matu'
cmeUnknown = u'Nezn\xe1m\xfd'
cmsRead = u'Precteno'
cmsReceived = u'Prijato'
cmsSending = u'Odes\xedl\xe1m...'
cmsSent = u'Odesl\xe1no'
cmsUnknown = u'Nezn\xe1m\xfd'
conConnecting = u'Spojuji'
conOffline = u'Offline'
conOnline = u'Online'
conPausing = u'Pozastavov\xe1n\xed'
conUnknown = u'Nezn\xe1m\xfd'
cusAway = u'Nepr\xedtomn\xfd'
cusDoNotDisturb = u'Neru\u0161it'
cusInvisible = u'Neviditeln\xfd'
cusLoggedOut = u'Offline'
cusNotAvailable = u'Nedostupn\xfd'
cusOffline = u'Offline'
cusOnline = u'Online'
cusSkypeMe = u'Skype Me'
cusUnknown = u'Nezn\xe1m\xfd'
cvsBothEnabled = u'Odes\xedl\xe1n\xed a pr\xedjem videa'
cvsNone = u'Bez videa'
cvsReceiveEnabled = u'Pr\xedjem videa'
cvsSendEnabled = u'Odes\xedl\xe1n\xed videa'
cvsUnknown = u''
grpAllFriends = u'V\u0161ichni pr\xe1tel\xe9'
grpAllUsers = u'V\u0161ichni u\u017eivatel\xe9'
grpCustomGroup = u'Vlastn\xed'
grpOnlineFriends = u'Pr\xe1tel\xe9 online'
grpPendingAuthorizationFriends = u'Nevyr\xedzen\xe1 autorizace'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Ned\xe1vno kontaktovan\xed u\u017eivatel\xe9'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Pr\xe1tel\xe9 pou\u017e\xedvaj\xedc\xed Skype'
grpSkypeOutFriends = u'Pr\xe1tel\xe9 pou\u017e\xedvaj\xedc\xed SkypeOut'
grpUngroupedFriends = u'Nezarazen\xed pr\xe1tel\xe9'
grpUnknown = u'Nezn\xe1m\xfd'
grpUsersAuthorizedByMe = u'Autorizovan\xed'
grpUsersBlockedByMe = u'Blokovan\xed'
grpUsersWaitingMyAuthorization = u'Cekaj\xedc\xed na autorizaci'
leaAddDeclined = u'Prid\xe1n\xed bylo odm\xedtnuto.'
leaAddedNotAuthorized = u'Prid\xe1van\xfd mus\xed b\xfdt autorizov\xe1n.'
leaAdderNotFriend = u'Prid\xe1vaj\xedc\xed mus\xed b\xfdt pr\xedtel.'
leaUnknown = u'Nezn\xe1m\xfd'
leaUnsubscribe = u'Odebran\xfd ze seznamu'
leaUserIncapable = u'U\u017eivatel je nezpusobil\xfd.'
leaUserNotFound = u'U\u017eivatel nebyl nalezen.'
olsAway = u'Nepr\xedtomn\xfd'
olsDoNotDisturb = u'Neru\u0161it'
olsNotAvailable = u'Nedostupn\xfd'
olsOffline = u'Offline'
olsOnline = u'Online'
olsSkypeMe = u'Skype Me'
olsSkypeOut = u'????????? ?? SkypeOut'
olsUnknown = u'Nezn\xe1m\xfd'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'\u017dena'
usexMale = u'Mu\u017e'
usexUnknown = u'Nezn\xe1m\xfd'
vmrConnectError = u'Chyba pripojen\xed'
vmrFileReadError = u'Chyba cten\xed souboru'
vmrFileWriteError = u'Chyba z\xe1pisu souboru'
vmrMiscError = u'Jin\xe1 chyba'
vmrNoError = u'Bez chyby'
vmrNoPrivilege = u'Nem\xe1te opr\xe1vnen\xed k hlasov\xe9 schr\xe1nce.'
vmrNoVoicemail = u'Takov\xe1 hlasov\xe1 schr\xe1nka neexistuje.'
vmrPlaybackError = u'Chyba prehr\xe1v\xe1n\xed'
vmrRecordingError = u'Chyba nahr\xe1v\xe1n\xed'
vmrUnknown = u'Nezn\xe1m\xfd'
vmsBlank = u'Pr\xe1zdn\xe9'
vmsBuffering = u'Nac\xedt\xe1n\xed'
vmsDeleting = u'Odstranov\xe1n\xed'
vmsDownloading = u'Stahov\xe1n\xed'
vmsFailed = u'Ne\xfaspe\u0161n\xe9'
vmsNotDownloaded = u'Nesta\u017eeno.'
vmsPlayed = u'Prehr\xe1no'
vmsPlaying = u'Prehr\xe1v\xe1n\xed'
vmsRecorded = u'Nahr\xe1no'
vmsRecording = u'Nahr\xe1v\xe1n\xed hlasov\xe9 zpr\xe1vy'
vmsUnknown = u'Nezn\xe1m\xfd'
vmsUnplayed = u'Neprehr\xe1no'
vmsUploaded = u'Odesl\xe1no'
vmsUploading = u'Odes\xedl\xe1n\xed'
vmtCustomGreeting = u'Vlastn\xed pozdrav'
vmtDefaultGreeting = u'V\xfdchoz\xed pozdrav'
vmtIncoming = u'pr\xedchoz\xed hlasov\xe1 zpr\xe1va'
vmtOutgoing = u'Odchoz\xed'
vmtUnknown = u'Nezn\xe1m\xfd'
vssAvailable = u'Dostupn\xfd'
vssNotAvailable = u'Nedostupn\xfd'
vssPaused = u'Pozastaveno'
vssRejected = u'Odm\xedtnuto'
vssRunning = u'Prob\xedh\xe1'
vssStarting = u'Start'
vssStopping = u'Ukoncov\xe1n\xed'
vssUnknown = u'Nezn\xe1m\xfd'



apiAttachAvailable = u'API jest dost\u0119pne'
apiAttachNotAvailable = u'Niedost\u0119pny'
apiAttachPendingAuthorization = u'Autoryzacja w toku'
apiAttachRefused = u'Odmowa'
apiAttachSuccess = u'Sukces'
apiAttachUnknown = u'Nieznany'
budDeletedFriend = u'Usuni\u0119ty z listy znajomych'
budFriend = u'Znajomy'
budNeverBeenFriend = u'Nigdy nie by\u0142 na li\u015bcie znajomych'
budPendingAuthorization = u'Autoryzacja w toku'
budUnknown = u'Nieznany'
cfrBlockedByRecipient = u'Po\u0142\u0105czenie zablokowane przez odbiorc\u0119'
cfrMiscError = u'B\u0142\u0105d'
cfrNoCommonCodec = u'Brak podstawowego kodeka'
cfrNoProxyFound = u'Nie odnaleziono serwera proksy'
cfrNotAuthorizedByRecipient = u'Ten u\u017cytkownik nie ma autoryzacji odbiorcy'
cfrRecipientNotFriend = u'Odbiorca nie jest znajomym'
cfrRemoteDeviceError = u'Problem ze zdalnym urz\u0105dzeniem d\u017awi\u0119kowym'
cfrSessionTerminated = u'Sesja zako\u0144czona'
cfrSoundIOError = u'B\u0142\u0105d d\u017awi\u0119ku przychodz\u0105cego lub wychodz\u0105cego'
cfrSoundRecordingError = u'B\u0142\u0105d nagrywania d\u017awi\u0119ku'
cfrUnknown = u'Nieznany'
cfrUserDoesNotExist = u'Taki u\u017cytkownik lub numer telefonu nie istnieje'
cfrUserIsOffline = u'Ona lub On jest niedost\u0119pny'
chsAllCalls = u'Wszystkie'
chsDialog = u'Dialog'
chsIncomingCalls = u'Zaakceptuj wielu uczestnik\xf3w'
chsLegacyDialog = u'Dialog przestarza\u0142y'
chsMissedCalls = u'Nie odebrane'
chsMultiNeedAccept = u'Zaakceptuj wielu uczestnik\xf3w'
chsMultiSubscribed = u'Wielu subskrybowanych'
chsOutgoingCalls = u'Wielu subskrybowanych'
chsUnknown = u'Nieznany'
chsUnsubscribed = u'Nie jest abonentem'
clsBusy = u'Zaj\u0119te'
clsCancelled = u'Anulowane'
clsEarlyMedia = u'Odtwarzanie wczesnych medi\xf3w (Early Media)'
clsFailed = u'Niestety, nieudane po\u0142\u0105czenie!'
clsFinished = u'Zako\u0144czono'
clsInProgress = u'Rozmowa w toku'
clsLocalHold = u'Zawieszona przez u\u017cytkownika'
clsMissed = u'Nieodebrana rozmowa'
clsOnHold = u'Zawieszona'
clsRefused = u'Odmowa'
clsRemoteHold = u'Zawieszona przez odbiorc\u0119'
clsRinging = u'Dzwoni'
clsRouting = u'Trasowanie'
clsTransferred = u'Nieznany'
clsTransferring = u'Nieznany'
clsUnknown = u'Nieznany'
clsUnplaced = u'Nigdy nie \u0142aczono'
clsVoicemailBufferingGreeting = u'Pozdrowienia podczas buforowania'
clsVoicemailCancelled = u'Poczta g\u0142osowa anulowana'
clsVoicemailFailed = u'B\u0142\u0105d poczty g\u0142osowej'
clsVoicemailPlayingGreeting = u'Odtwarzanie pozdrowienia'
clsVoicemailRecording = u'Nagrywanie poczty g\u0142osowej'
clsVoicemailSent = u'Poczta g\u0142osowa wys\u0142ana'
clsVoicemailUploading = u'Wysy\u0142anie poczty g\u0142osowej'
cltIncomingP2P = u'Rozmowa przychodz\u0105ca peer-to-peer'
cltIncomingPSTN = u'Rozmowa przychodz\u0105ca'
cltOutgoingP2P = u'Rozmowa wychodz\u0105ca peer-to-peer'
cltOutgoingPSTN = u'Rozmowa wychodz\u0105ca'
cltUnknown = u'Nieznany'
cmeAddedMembers = u'Cz\u0142onkowie dodani'
cmeCreatedChatWith = u'Rozpocz\u0119ty czat z'
cmeEmoted = u'Emoted'
cmeLeft = u'Opusci\u0142'
cmeSaid = u'Powiedzia\u0142'
cmeSawMembers = u'Zobaczy\u0142e\u015b cz\u0142onk\xf3w'
cmeSetTopic = u'Ustaw temat'
cmeUnknown = u'Nieznany'
cmsRead = u'Przeczyta\u0142'
cmsReceived = u'Otrzyma\u0142'
cmsSending = u'Wysy\u0142am...'
cmsSent = u'Wys\u0142any'
cmsUnknown = u'Nieznany'
conConnecting = u'\u0141\u0105czenie'
conOffline = u'Niepod\u0142\u0105czony'
conOnline = u'Dost\u0119pny'
conPausing = u'Wstrzymane'
conUnknown = u'Nieznany'
cusAway = u'Zaraz wracam'
cusDoNotDisturb = u'Nie przeszkadza\u0107'
cusInvisible = u'Niewidoczny'
cusLoggedOut = u'Niepod\u0142\u0105czony'
cusNotAvailable = u'Niedost\u0119pny'
cusOffline = u'Niepod\u0142\u0105czony'
cusOnline = u'Dost\u0119pny'
cusSkypeMe = u"Tryb 'Skype Me'"
cusUnknown = u'Nieznany'
cvsBothEnabled = u'Odbierz i odbierz wideo'
cvsNone = u'Bez wideo'
cvsReceiveEnabled = u'Odbierz wideo'
cvsSendEnabled = u'Wy\u015blij wideo'
cvsUnknown = u'Nieznany'
grpAllFriends = u'Wszyscy znajomi'
grpAllUsers = u'Wszyscy u\u017cytkownicy'
grpCustomGroup = u'Niestandardowe'
grpOnlineFriends = u'Znajomi w sieci'
grpPendingAuthorizationFriends = u'Autoryzacja w toku'
grpProposedSharedGroup = u'Propozycja grupy wsp\xf3\u0142dzielonej'
grpRecentlyContactedUsers = u'Ostatnie kontakty'
grpSharedGroup = u'Wsp\xf3\u0142dzielona grupa'
grpSkypeFriends = u'Znajomi ze Skype'
grpSkypeOutFriends = u'Znajomi ze SkypeOut'
grpUngroupedFriends = u'Znajomi spoza grupy'
grpUnknown = u'Nieznany'
grpUsersAuthorizedByMe = u'Moja autoryzacja'
grpUsersBlockedByMe = u'Moja blokada'
grpUsersWaitingMyAuthorization = u'Pro\u015bba o autoryzacj\u0119'
leaAddDeclined = u'Dodawanie odrzucone'
leaAddedNotAuthorized = u'Osoba dodawana musi by\u0107 autoryzowana'
leaAdderNotFriend = u'Osoba dodaj\u0105ca musi by\u0107 znajomym'
leaUnknown = u'Nieznany'
leaUnsubscribe = u'Nie jest abonentem'
leaUserIncapable = u'U\u017cytkownik nie mo\u017ce rozmawia\u0107'
leaUserNotFound = u'U\u017cytkownik nie zosta\u0142 znaleziony'
olsAway = u'Zaraz wracam'
olsDoNotDisturb = u'Nie przeszkadza\u0107'
olsNotAvailable = u'Niedost\u0119pny'
olsOffline = u'Niepod\u0142\u0105czony'
olsOnline = u'Dost\u0119pny'
olsSkypeMe = u"Tryb 'Skype Me'"
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Nieznany'
smsMessageStatusComposing = u'Tworzenie'
smsMessageStatusDelivered = u'Dor\u0119czona'
smsMessageStatusFailed = u'Nieudane'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Otrzymany'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Wys\u0142ana do serwera'
smsMessageStatusSomeTargetsFailed = u'Niekt\xf3re numery nieudane'
smsMessageStatusUnknown = u'Nieznany'
smsMessageTypeCCRequest = u'Pro\u015bba o kod potwierdzaj\u0105cy'
smsMessageTypeCCSubmit = u'Wys\u0142anie kodu potwierdzaj\u0105cego'
smsMessageTypeIncoming = u'Przychodz\u0105ca'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Akceptowalny'
smsTargetStatusAnalyzing = u'Analiza'
smsTargetStatusDeliveryFailed = u'Nieudane'
smsTargetStatusDeliveryPending = u'Oczekuje'
smsTargetStatusDeliverySuccessful = u'Dor\u0119czona'
smsTargetStatusNotRoutable = u'Brak trasy'
smsTargetStatusUndefined = u'Niezdefiniowana'
smsTargetStatusUnknown = u'Nieznany'
usexFemale = u'Kobieta'
usexMale = u'M\u0119\u017cczyzna'
usexUnknown = u'Nieznany'
vmrConnectError = u'B\u0142\u0105d po\u0142\u0105czenia'
vmrFileReadError = u'B\u0142\u0105d odczytu pliku'
vmrFileWriteError = u'B\u0142\u0105d zapisu pliku'
vmrMiscError = u'B\u0142\u0105d'
vmrNoError = u'Bez b\u0142\u0119du'
vmrNoPrivilege = u'Brak uprawnie\u0144 Voicemail'
vmrNoVoicemail = u'Taka poczta g\u0142osowa nie istnieje'
vmrPlaybackError = u'B\u0142\u0105d odtwarzania'
vmrRecordingError = u'B\u0142\u0105d nagrywania'
vmrUnknown = u'Nieznany'
vmsBlank = u'Pusty'
vmsBuffering = u'Buforowanie'
vmsDeleting = u'Usuwanie'
vmsDownloading = u'Trwa pobieranie'
vmsFailed = u'Nie powiodlo si\u0119'
vmsNotDownloaded = u'Niepobrany'
vmsPlayed = u'Odtworzony'
vmsPlaying = u'Odtwarzanie'
vmsRecorded = u'Nagrany'
vmsRecording = u'Nagrywanie poczty g\u0142osowej'
vmsUnknown = u'Nieznany'
vmsUnplayed = u'Nieodtworzony'
vmsUploaded = u'Przekazany'
vmsUploading = u'Przekazywanie'
vmtCustomGreeting = u'Pozdrowienia niestandardowe'
vmtDefaultGreeting = u'Pozdrowienia domy\u015blne'
vmtIncoming = u'przysy\u0142ana jest wiadomo\u015b\u0107 g\u0142osowa'
vmtOutgoing = u'Wychodz\u0105ca'
vmtUnknown = u'Nieznany'
vssAvailable = u'Dost\u0119pny'
vssNotAvailable = u'Niedostepny'
vssPaused = u'Wstrzymane'
vssRejected = u'Odrzucona'
vssRunning = u'Trwaj\u0105ca'
vssStarting = u'Rozpocz\u0119cie'
vssStopping = u'Zatrzymanie'
vssUnknown = u'Nieznany'



apiAttachAvailable = u'API disponibile'
apiAttachNotAvailable = u'Non disponibile'
apiAttachPendingAuthorization = u'In attesa di autorizzazione'
apiAttachRefused = u'Rifiutato'
apiAttachSuccess = u'Riuscito'
apiAttachUnknown = u'Sconosciuto'
budDeletedFriend = u"Eliminato dall'elenco amici"
budFriend = u'Amico'
budNeverBeenFriend = u'Mai stato in elenco amici'
budPendingAuthorization = u'In attesa di autorizzazione'
budUnknown = u'Sconosciuto'
cfrBlockedByRecipient = u'Chiamata bloccata dal destinatario'
cfrMiscError = u'Errori vari'
cfrNoCommonCodec = u'Nessun codec comune'
cfrNoProxyFound = u'Nessun proxy trovato'
cfrNotAuthorizedByRecipient = u'Utente corrente non autorizzato dal destinatario'
cfrRecipientNotFriend = u'Il destinatario non \xe8 un amico'
cfrRemoteDeviceError = u'Problema con la periferica audio remota'
cfrSessionTerminated = u'Sessione conclusa'
cfrSoundIOError = u'Errore I/O audio'
cfrSoundRecordingError = u'Errore di registrazione audio'
cfrUnknown = u'Sconosciuto'
cfrUserDoesNotExist = u'Utente o numero di telefono inesistente'
cfrUserIsOffline = u'Non \xe8 in linea'
chsAllCalls = u'Finestra versione'
chsDialog = u'Dialogo'
chsIncomingCalls = u'In attesa di riscontro'
chsLegacyDialog = u'Finestra versione'
chsMissedCalls = u'Dialogo'
chsMultiNeedAccept = u'In attesa di riscontro'
chsMultiSubscribed = u'Multi-iscritti'
chsOutgoingCalls = u'Multi-iscritti'
chsUnknown = u'Sconosciuto'
chsUnsubscribed = u'Disiscritto'
clsBusy = u'Occupato'
clsCancelled = u'Cancellato'
clsEarlyMedia = u'Esecuzione Early Media in corso (Playing Early Media)'
clsFailed = u'spiacente, chiamata non riuscita!'
clsFinished = u'Terminata'
clsInProgress = u'Chiamata in corso'
clsLocalHold = u'Chiamata in sospeso da utente locale'
clsMissed = u'Chiamata persa'
clsOnHold = u'Sospesa'
clsRefused = u'Rifiutato'
clsRemoteHold = u'Chiamata in sospeso da utente remoto'
clsRinging = u'in chiamata'
clsRouting = u'Routing'
clsTransferred = u'Sconosciuto'
clsTransferring = u'Sconosciuto'
clsUnknown = u'Sconosciuto'
clsUnplaced = u'Mai effettuata'
clsVoicemailBufferingGreeting = u'Buffering del saluto in corso'
clsVoicemailCancelled = u'La voicemail \xe8 stata annullata'
clsVoicemailFailed = u'Messaggio vocale non inviato'
clsVoicemailPlayingGreeting = u'Esecuzione saluto'
clsVoicemailRecording = u'Registrazione del messaggio vocale'
clsVoicemailSent = u'La voicemail \xe8 stata inviata'
clsVoicemailUploading = u'Caricamento voicemail in corso'
cltIncomingP2P = u'Chiamata Peer-to-Peer in arrivo'
cltIncomingPSTN = u'Telefonata in arrivo'
cltOutgoingP2P = u'Chiamata Peer-to-Peer in uscita'
cltOutgoingPSTN = u'Telefonata in uscita'
cltUnknown = u'Sconosciuto'
cmeAddedMembers = u'Membri aggiunti'
cmeCreatedChatWith = u'Chat creata con'
cmeEmoted = u'Sconosciuto'
cmeLeft = u'Uscito'
cmeSaid = u'Detto'
cmeSawMembers = u'Membri visti'
cmeSetTopic = u'Argomento impostato'
cmeUnknown = u'Sconosciuto'
cmsRead = u'Letto'
cmsReceived = u'Ricevuto'
cmsSending = u'Sto inviando...'
cmsSent = u'Inviato'
cmsUnknown = u'Sconosciuto'
conConnecting = u'In connessione'
conOffline = u'Non in linea'
conOnline = u'In linea'
conPausing = u'In pausa'
conUnknown = u'Sconosciuto'
cusAway = u'Torno subito'
cusDoNotDisturb = u'Occupato'
cusInvisible = u'Invisibile'
cusLoggedOut = u'Non in linea'
cusNotAvailable = u'Non disponibile'
cusOffline = u'Non in linea'
cusOnline = u'In linea'
cusSkypeMe = u'Libero per la Chat'
cusUnknown = u'Sconosciuto'
cvsBothEnabled = u'Invio e ricezione video'
cvsNone = u'Assenza video'
cvsReceiveEnabled = u'Ricezione video'
cvsSendEnabled = u'Invio video'
cvsUnknown = u''
grpAllFriends = u'Tutti gli amici'
grpAllUsers = u'Tutti gli utenti'
grpCustomGroup = u'Personalizzato'
grpOnlineFriends = u'Amici online'
grpPendingAuthorizationFriends = u'In attesa di autorizzazione'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Utenti contattati di recente'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Amici Skype'
grpSkypeOutFriends = u'Amici SkypeOut'
grpUngroupedFriends = u'Amici non di gruppo'
grpUnknown = u'Sconosciuto'
grpUsersAuthorizedByMe = u'Autorizzato da me'
grpUsersBlockedByMe = u'Bloccato da me'
grpUsersWaitingMyAuthorization = u'In attesa di mia autorizzazione'
leaAddDeclined = u'Rifiutato'
leaAddedNotAuthorized = u'Deve essere autorizzato'
leaAdderNotFriend = u'Deve essere un amico'
leaUnknown = u'Sconosciuto'
leaUnsubscribe = u'Disiscritto'
leaUserIncapable = u'Utente incapace'
leaUserNotFound = u'Utente non trovato'
olsAway = u'Torno subito'
olsDoNotDisturb = u'Occupato'
olsNotAvailable = u'Non disponibile'
olsOffline = u'Non in linea'
olsOnline = u'In linea'
olsSkypeMe = u'Libero per la Chat'
olsSkypeOut = u'SkypeOut...'
olsUnknown = u'Sconosciuto'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Femmina'
usexMale = u'Maschio'
usexUnknown = u'Sconosciuto'
vmrConnectError = u'Errore di connessione'
vmrFileReadError = u'Errore lettura file'
vmrFileWriteError = u'Errore scrittura file'
vmrMiscError = u'Errori vari'
vmrNoError = u'Nessun errore'
vmrNoPrivilege = u'Nessun privilegio voicemail'
vmrNoVoicemail = u'Voicemail inesistente'
vmrPlaybackError = u'Errore di riproduzione'
vmrRecordingError = u'Errore di registrazione'
vmrUnknown = u'Sconosciuto'
vmsBlank = u'Vuota'
vmsBuffering = u'Buffering'
vmsDeleting = u'Eliminazione in corso'
vmsDownloading = u'Download in corso'
vmsFailed = u'Fallita'
vmsNotDownloaded = u'Non scaricata'
vmsPlayed = u'Riprodotta'
vmsPlaying = u'Riproduzione in corso'
vmsRecorded = u'Registrata'
vmsRecording = u'Registrazione del messaggio vocale'
vmsUnknown = u'Sconosciuto'
vmsUnplayed = u'Non riprodotta'
vmsUploaded = u'Caricata'
vmsUploading = u'Caricamento in corso'
vmtCustomGreeting = u'Saluto personalizzato'
vmtDefaultGreeting = u'Saluto predefinito'
vmtIncoming = u'Messaggio vocale in arrivo'
vmtOutgoing = u'In uscita'
vmtUnknown = u'Sconosciuto'
vssAvailable = u'Disponibile'
vssNotAvailable = u'Non disponibile'
vssPaused = u'In pausa'
vssRejected = u'Rifiutata'
vssRunning = u'In corso'
vssStarting = u'Avvio in corso'
vssStopping = u'Arresto in corso'
vssUnknown = u'Sconosciuto'



"""
Low level *Skype for Windows* interface implemented using *Windows messaging*.
Uses direct *WinAPI* calls through *ctypes* module.

This module handles the options that you can pass to `Skype.__init__`
for Windows machines.

No options are currently supported.
"""
__docformat__ = 'restructuredtext en'


import sys
import threading
import time
from ctypes import *
import logging

from Skype4Py.api import Command, SkypeAPIBase, \
                         timeout2float, finalize_opts, \
                         DEFAULT_TIMEOUT
from Skype4Py.enums import *
from Skype4Py.errors import SkypeAPIError


__all__ = ['SkypeAPI']


try:
    WNDPROC = WINFUNCTYPE(c_long, c_int, c_uint, c_int, c_int)
except NameError:
    # Proceed only if our setup.py is not running.
    if not getattr(sys, 'skype4py_setup', False):
        raise
    # This will allow importing of this module on non-Windows machines. It won't work
    # of course but this will allow building documentation on any platform.
    WNDPROC = c_void_p


class WNDCLASS(Structure):
    _fields_ = [('style', c_uint),
                ('lpfnWndProc', WNDPROC),
                ('cbClsExtra', c_int),
                ('cbWndExtra', c_int),
                ('hInstance', c_int),
                ('hIcon', c_int),
                ('hCursor', c_int),
                ('hbrBackground', c_int),
                ('lpszMenuName', c_char_p),
                ('lpszClassName', c_char_p)]


class MSG(Structure):
    _fields_ = [('hwnd', c_int),
                ('message', c_uint),
                ('wParam', c_int),
                ('lParam', c_int),
                ('time', c_int),
                ('pointX', c_long),
                ('pointY', c_long)]


class COPYDATASTRUCT(Structure):
    _fields_ = [('dwData', POINTER(c_uint)),
                ('cbData', c_uint),
                ('lpData', c_char_p)]


PCOPYDATASTRUCT = POINTER(COPYDATASTRUCT)

WM_QUIT = 0x12
WM_COPYDATA = 0x4A

HWND_BROADCAST = 0xFFFF


class SkypeAPI(SkypeAPIBase):
    def __init__(self, opts):
        self.logger = logging.getLogger('Skype4Py.api.windows.SkypeAPI')
        SkypeAPIBase.__init__(self)
        finalize_opts(opts)
        self.window_class = None
        self.hwnd = None
        self.skype = None
        self.wait = False
        self.SkypeControlAPIDiscover = windll.user32.RegisterWindowMessageA('SkypeControlAPIDiscover')
        self.SkypeControlAPIAttach = windll.user32.RegisterWindowMessageA('SkypeControlAPIAttach')
        windll.user32.GetWindowLongA.restype = c_ulong

    def run(self):
        self.logger.info('thread started')
        if not self.create_window():
            self.hwnd = None
            return

        msg = MSG()
        pmsg = pointer(msg)
        while self.hwnd and windll.user32.GetMessageA(pmsg, self.hwnd, 0, 0):
            windll.user32.TranslateMessage(pmsg)
            windll.user32.DispatchMessageA(pmsg)

        self.destroy_window()
        self.hwnd = None
        self.logger.info('thread finished')

    def close(self):
        if self.hwnd:
            windll.user32.PostMessageA(self.hwnd, WM_QUIT, 0, 0)
            while self.hwnd:
                time.sleep(0.01)
        self.skype = None
        SkypeAPIBase.close(self)

    def set_friendly_name(self, friendly_name):
        SkypeAPIBase.set_friendly_name(self, friendly_name)
        if self.skype:
            self.send_command(Command('NAME %s' % friendly_name))

    def get_foreground_window(self):
        fhwnd = windll.user32.GetForegroundWindow()
        if fhwnd:
            # awahlig (7.05.2008):
            # I've found at least one app (RocketDock) that had window style 8 set.
            # This is odd since windows header files do not contain such a style.
            # Doing message exchange while this window is a foreground one, causes
            # lockups if some operations on client UI are involved (for example
            # sending a 'FOCUS' command). Therefore, we will set our window as
            # the foreground one for the transmission time.
            if windll.user32.GetWindowLongA(fhwnd, -16) & 8 == 0:
                fhwnd = None
        return fhwnd
        
    def attach(self, timeout, wait=True):
        if self.skype is not None and windll.user32.IsWindow(self.skype):
            return
        self.acquire()
        self.skype = None
        try:
            if not self.isAlive():
                try:
                    self.start()
                except AssertionError:
                    raise SkypeAPIError('Skype API closed')
                # wait till the thread initializes
                while not self.hwnd:
                    time.sleep(0.01)
            self.logger.debug('broadcasting SkypeControlAPIDiscover')
            fhwnd = self.get_foreground_window()
            try:
                if fhwnd:
                    windll.user32.SetForegroundWindow(self.hwnd)
                if not windll.user32.SendMessageTimeoutA(HWND_BROADCAST, self.SkypeControlAPIDiscover,
                                                         self.hwnd, None, 2, 5000, None):
                    raise SkypeAPIError('Could not broadcast Skype discover message')
                # wait (with timeout) till the WindProc() attaches
                self.wait = True
                t = threading.Timer(timeout2float(timeout), lambda: setattr(self, 'wait', False))
                if wait:
                    t.start()
                while self.wait and self.attachment_status not in (apiAttachSuccess, apiAttachRefused):
                    if self.attachment_status == apiAttachPendingAuthorization:
                        # disable the timeout
                        t.cancel()
                    elif self.attachment_status == apiAttachAvailable:
                        # rebroadcast
                        self.logger.debug('broadcasting SkypeControlAPIDiscover')
                        windll.user32.SetForegroundWindow(self.hwnd)
                        if not windll.user32.SendMessageTimeoutA(HWND_BROADCAST, self.SkypeControlAPIDiscover,
                                                                 self.hwnd, None, 2, 5000, None):
                            raise SkypeAPIError('Could not broadcast Skype discover message')
                    time.sleep(0.01)
                t.cancel()
            finally:
                if fhwnd:
                    windll.user32.SetForegroundWindow(fhwnd)
        finally:
            self.release()
        # check if we got the Skype window's hwnd
        if self.skype is not None:
            command = Command('PROTOCOL %s' % self.protocol, Blocking=True)
            self.send_command(command)
            self.protocol = int(command.Reply.rsplit(None, 1)[-1])
        elif not self.wait:
            raise SkypeAPIError('Skype attach timeout')

    def is_running(self):
        # tSkMainForm is for Skype 5-6, TZap for 4.0, tSk.UnicodeClass for 3.8
        return bool(windll.user32.FindWindowA('tSkMainForm', None)
                    or windll.user32.FindWindowA('TZapMainForm.UnicodeClass', None)
                    or windll.user32.FindWindowA('tSkMainForm.UnicodeClass', None))

    def get_skype_path(self):
        key = c_long()
        # try to find Skype in HKEY_CURRENT_USER registry tree
        if windll.advapi32.RegOpenKeyA(0x80000001, 'Software\\Skype\\Phone', byref(key)) != 0:
            # try to find Skype in HKEY_LOCAL_MACHINE registry tree
            if windll.advapi32.RegOpenKeyA(0x80000002, 'Software\\Skype\\Phone', byref(key)) != 0:
                raise SkypeAPIError('Skype not installed')
        pathlen = c_long(512)
        path = create_string_buffer(pathlen.value)
        if windll.advapi32.RegQueryValueExA(key, 'SkypePath', None, None, path, byref(pathlen)) != 0:
            windll.advapi32.RegCloseKey(key)
            raise SkypeAPIError('Cannot find Skype path')
        windll.advapi32.RegCloseKey(key)
        return path.value

    def startup(self, minimized, nosplash):
        args = []
        if minimized:
            args.append('/MINIMIZED')
        if nosplash:
            args.append('/NOSPLASH')
        try:
            if self.hwnd:
                fhwnd = self.get_foreground_window()
                if fhwnd:
                    windll.user32.SetForegroundWindow(self.hwnd)
            if windll.shell32.ShellExecuteA(None, 'open', self.get_skype_path(), ' '.join(args), None, 0) <= 32:
                raise SkypeAPIError('Could not start Skype')
        finally:
            if self.hwnd and fhwnd:
                windll.user32.SetForegroundWindow(fhwnd)
        
    def shutdown(self):
        try:
            if self.hwnd:
                fhwnd = self.get_foreground_window()
                if fhwnd:
                    windll.user32.SetForegroundWindow(self.hwnd)
            if windll.shell32.ShellExecuteA(None, 'open', self.get_skype_path(), '/SHUTDOWN', None, 0) <= 32:
                raise SkypeAPIError('Could not shutdown Skype')
        finally:
            if self.hwnd and fhwnd:
                windll.user32.SetForegroundWindow(fhwnd)

    def create_window(self):
        # window class has to be saved as property to keep reference to self.WinProc
        self.window_class = WNDCLASS(3, WNDPROC(self.window_proc), 0, 0,
                                     windll.kernel32.GetModuleHandleA(None),
                                     0, 0, 0, None, 'Skype4Py.%d' % id(self))

        wclass = windll.user32.RegisterClassA(byref(self.window_class))
        if wclass == 0:
            return False

        self.hwnd = windll.user32.CreateWindowExA(0, 'Skype4Py.%d' % id(self), 'Skype4Py',
                                                  0xCF0000, 0x80000000, 0x80000000,
                                                  0x80000000, 0x80000000, None, None,
                                                  self.window_class.hInstance, 0)
        if self.hwnd == 0:
            windll.user32.UnregisterClassA('Skype4Py.%d' % id(self), None)
            return False

        return True

    def destroy_window(self):
        if not windll.user32.DestroyWindow(self.hwnd):
            return False
        self.hwnd = None

        if not windll.user32.UnregisterClassA('Skype4Py.%d' % id(self), None):
            return False
        self.window_class = None

        return True

    def window_proc(self, hwnd, umsg, wparam, lparam):
        if umsg == self.SkypeControlAPIAttach:
            self.logger.debug('received SkypeControlAPIAttach %s', lparam)
            if lparam == apiAttachSuccess:
                if self.skype is None or self.skype == wparam:
                    self.skype = wparam
                else:
                    self.logger.warning('second successful attach received for different API window')
            elif lparam in (apiAttachRefused, apiAttachNotAvailable, apiAttachAvailable):
                self.skype = None
            elif lparam == apiAttachPendingAuthorization:
                if self.attachment_status == apiAttachSuccess:
                    self.logger.warning('received pending attach after successful attach')
                    return 0
            self.set_attachment_status(lparam)
            return 1
        elif umsg == WM_COPYDATA and wparam == self.skype and lparam:
            copydata = cast(lparam, PCOPYDATASTRUCT).contents
            cmd8 = copydata.lpData[:copydata.cbData - 1]
            cmd = cmd8.decode('utf-8')
            self.logger.debug('received %s', repr(cmd))
            if cmd.startswith(u'#'):
                p = cmd.find(u' ')
                command = self.pop_command(int(cmd[1:p]))
                if command is not None:
                    command.Reply = cmd[p + 1:]
                    if command.Blocking:
                        command._event.set()
                    else:
                        command._timer.cancel()
                    self.notifier.reply_received(command)
                else:
                    self.notifier.notification_received(cmd[p + 1:])
            else:
                self.notifier.notification_received(cmd)
            return 1
        elif umsg == apiAttachAvailable:
            self.logger.debug('received apiAttachAvailable')
            self.skype = None
            self.set_attachment_status(umsg)
            return 1
        return windll.user32.DefWindowProcA(c_int(hwnd), c_int(umsg), c_int(wparam), c_int(lparam))

    def send_command(self, command):
        for retry in xrange(2):
            if self.skype is None:
                self.attach(command.Timeout)
            self.push_command(command)
            self.notifier.sending_command(command)
            cmd = u'#%d %s' % (command.Id, command.Command)
            cmd8 = cmd.encode('utf-8') + '\0'
            copydata = COPYDATASTRUCT(None, len(cmd8), cmd8)
            if command.Blocking:
                command._event = event = threading.Event()
            else:
                command._timer = timer = threading.Timer(command.timeout2float(), self.pop_command, (command.Id,))
            self.logger.debug('sending %s', repr(cmd))
            fhwnd = self.get_foreground_window()
            try:
                if fhwnd:
                    windll.user32.SetForegroundWindow(self.hwnd)
                if windll.user32.SendMessageA(self.skype, WM_COPYDATA, self.hwnd, byref(copydata)):
                    if command.Blocking:
                        event.wait(command.timeout2float())
                        if not event.isSet():
                            raise SkypeAPIError('Skype command timeout')
                    else:
                        timer.start()
                    break
                else:
                    # SendMessage failed
                    self.pop_command(command.Id)
                    self.skype = None
                    # let the loop go back and try to reattach but only once
            finally:
                if fhwnd:
                    windll.user32.SetForegroundWindow(fhwnd)
        else:
            raise SkypeAPIError('Skype API error, check if Skype wasn\'t closed')

    def allow_focus(self, timeout):
        if self.skype is None:
            self.attach(timeout)
        process_id = c_ulong()
        windll.user32.GetWindowThreadProcessId(self.skype, byref(process_id))
        if process_id:
            windll.user32.AllowSetForegroundWindow(process_id)



apiAttachAvailable = u'API disponibil'
apiAttachNotAvailable = u'Indisponibil'
apiAttachPendingAuthorization = u'Autorizare \xeen asteptare'
apiAttachRefused = u'Refuzat'
apiAttachSuccess = u'Conectare reusita'
apiAttachUnknown = u'Necunoscut'
budDeletedFriend = u'Elimina cifra 0 dinaintea prefixului local, folosit uneori pentru convorbiri interne. Acest lucru este valabil pentru Marea Britanie si Germania (si nu numai). Pentru apelurile \xeen Italia sau Rusia (si nu numai), cifra 0 trebuie sa ram\xe2na.'
budFriend = u'Prieten'
budNeverBeenFriend = u'Nu a fost niciodata \xeen Lista de prieteni'
budPendingAuthorization = u'Autorizare \xeen asteptare'
budUnknown = u'Necunoscut'
cfrBlockedByRecipient = u'Apel blocat de destinatar'
cfrMiscError = u'Eroare diversa'
cfrNoCommonCodec = u'No common codec found'
cfrNoProxyFound = u'Nu s-a gasit niciun proxy'
cfrNotAuthorizedByRecipient = u'Utilizatorul curent nu este autorizat de destinatar'
cfrRecipientNotFriend = u'Destinatarul nu are statutul de prieten'
cfrRemoteDeviceError = u'Problem with remote sound device'
cfrSessionTerminated = u'Session terminated'
cfrSoundIOError = u'Eroare I/O sunet'
cfrSoundRecordingError = u'Eroare \xeenregistrare sunet'
cfrUnknown = u'Necunoscut'
cfrUserDoesNotExist = u'Utilizator/numar de telefon inexistent'
cfrUserIsOffline = u'User is offline'
chsAllCalls = u'Dialog versiune anterioara'
chsDialog = u'Dialog'
chsIncomingCalls = u'\xcen asteptarea acceptarii de la mai multi utilizatori'
chsLegacyDialog = u'Dialog versiune anterioara'
chsMissedCalls = u'Dialog'
chsMultiNeedAccept = u'\xcen asteptarea acceptarii de la mai multi utilizatori'
chsMultiSubscribed = u'Mai multi utilizatori \xeenregistrati'
chsOutgoingCalls = u'Mai multi utilizatori \xeenregistrati'
chsUnknown = u'Necunoscut'
chsUnsubscribed = u'Neabonat'
clsBusy = u'Busy'
clsCancelled = u'Cancelled'
clsEarlyMedia = u'Redare elemente media premergatoare convorbirii(Early Media)'
clsFailed = u'Sorry, call failed!'
clsFinished = u'Terminat'
clsInProgress = u'Call in Progress'
clsLocalHold = u'\xcen asteptare local'
clsMissed = u'Missed'
clsOnHold = u'On Hold'
clsRefused = u'Refuzat'
clsRemoteHold = u'\xcen asteptare la distanta'
clsRinging = u'Calling'
clsRouting = u'Directionare'
clsTransferred = u'Necunoscut'
clsTransferring = u'Necunoscut'
clsUnknown = u'Necunoscut'
clsUnplaced = u'Apel neefectuat'
clsVoicemailBufferingGreeting = u'Utilizare zona tampon pentru mesaj de \xeent\xe2mpinare'
clsVoicemailCancelled = u'Mesajul vocal a fost revocat'
clsVoicemailFailed = u'Voicemail Failed'
clsVoicemailPlayingGreeting = u'Redare mesaj de \xeent\xe2mpinare'
clsVoicemailRecording = u'Recording'
clsVoicemailSent = u'Mesajul vocal a fost trimis'
clsVoicemailUploading = u'\xcencarcare mesaj vocal'
cltIncomingP2P = u'Apel Peer-to-Peer primit'
cltIncomingPSTN = u'Apel telefonic primit'
cltOutgoingP2P = u'Apel Peer-to-Peer efectuat'
cltOutgoingPSTN = u'Apel telefonic efectuat'
cltUnknown = u'Necunoscut'
cmeAddedMembers = u'Membri adaugati'
cmeCreatedChatWith = u'Chat creat cu'
cmeEmoted = u'Necunoscut'
cmeLeft = u'A parasit conferinta'
cmeSaid = u'A spus'
cmeSawMembers = u'Membri vazuti'
cmeSetTopic = u'Setare subiect'
cmeUnknown = u'Necunoscut'
cmsRead = u'Citit'
cmsReceived = u'Primit'
cmsSending = u'Sending...'
cmsSent = u'Trimis'
cmsUnknown = u'Necunoscut'
conConnecting = u'Connecting'
conOffline = u'Offline'
conOnline = u'Online'
conPausing = u'\xcen pauza'
conUnknown = u'Necunoscut'
cusAway = u'Away'
cusDoNotDisturb = u'Do Not Disturb'
cusInvisible = u'Invisible'
cusLoggedOut = u'Offline'
cusNotAvailable = u'Indisponibil'
cusOffline = u'Logged Out'
cusOnline = u'Online'
cusSkypeMe = u'Skype Me'
cusUnknown = u'Necunoscut'
cvsBothEnabled = u'Trimitere si primire video'
cvsNone = u'Fara video'
cvsReceiveEnabled = u'Primire video'
cvsSendEnabled = u'Trimitere video'
cvsUnknown = u''
grpAllFriends = u'Toti prietenii'
grpAllUsers = u'Toti utilizatorii'
grpCustomGroup = u'Personalizat'
grpOnlineFriends = u'Prieteni conectati'
grpPendingAuthorizationFriends = u'Autorizare \xeen asteptare'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Utilizatori contactati recent'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Prieteni Skype'
grpSkypeOutFriends = u'Prieteni SkypeOut'
grpUngroupedFriends = u'Prieteni negrupati'
grpUnknown = u'Necunoscut'
grpUsersAuthorizedByMe = u'Autorizati de mine'
grpUsersBlockedByMe = u'Blocati de mine'
grpUsersWaitingMyAuthorization = u'\xcen asteptarea autorizarii din partea mea'
leaAddDeclined = u'Adaugare refuzata'
leaAddedNotAuthorized = u'Utilizatorul adaugat trebuie sa fie autorizat'
leaAdderNotFriend = u'Utilizatorul care face adaugarea trebuie sa fie un prieten'
leaUnknown = u'Necunoscut'
leaUnsubscribe = u'Neabonat'
leaUserIncapable = u'Incapacitate utilizator'
leaUserNotFound = u'Nu s-a gasit utilizatorul'
olsAway = u'Away'
olsDoNotDisturb = u'Do Not Disturb'
olsNotAvailable = u'Indisponibil'
olsOffline = u'Offline'
olsOnline = u'Online'
olsSkypeMe = u'SkypeMe'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Necunoscut'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Female'
usexMale = u'Male'
usexUnknown = u'Necunoscut'
vmrConnectError = u'Eroare conectare'
vmrFileReadError = u'Eroare citire fisier'
vmrFileWriteError = u'Eroare scriere fisier'
vmrMiscError = u'Eroare diversa'
vmrNoError = u'Nicio eroare'
vmrNoPrivilege = u'Fara privilegii pentru mesaje vocale'
vmrNoVoicemail = u'Mesaj vocal inexistent'
vmrPlaybackError = u'Eroare redare'
vmrRecordingError = u'Eroare \xeenregistrare'
vmrUnknown = u'Necunoscut'
vmsBlank = u'Gol'
vmsBuffering = u'Utilizare zona tampon'
vmsDeleting = u'Stergere'
vmsDownloading = u'Descarcare'
vmsFailed = u'Nereusit'
vmsNotDownloaded = u'Nedescarcat'
vmsPlayed = u'Redat'
vmsPlaying = u'Redare'
vmsRecorded = u'\xcenregistrat'
vmsRecording = u'Recording Voicemail'
vmsUnknown = u'Necunoscut'
vmsUnplayed = u'Neredat'
vmsUploaded = u'\xcencarcat'
vmsUploading = u'\xcencarcare'
vmtCustomGreeting = u'Mesaj de \xeent\xe2mpinare personalizat'
vmtDefaultGreeting = u'Mesaj de \xeent\xe2mpinare implicit'
vmtIncoming = u'Incoming'
vmtOutgoing = u'De expediat'
vmtUnknown = u'Necunoscut'
vssAvailable = u'Disponibil'
vssNotAvailable = u'Indisponibil'
vssPaused = u'\xcen pauza'
vssRejected = u'Respins'
vssRunning = u'\xcen executie'
vssStarting = u'Lansare'
vssStopping = u'\xcentrerupere'
vssUnknown = u'Necunoscut'



"""Skype client settings.
"""
__docformat__ = 'restructuredtext en'


import sys
import weakref

from utils import *


class Settings(object):
    """Represents Skype settings. Access using `skype.Skype.Settings`.
    """

    def __init__(self, Skype):
        """__init__.

        :Parameters:
          Skype : `Skype`
            Skype
        """
        self._SkypeRef = weakref.ref(Skype)

    def Avatar(self, Id=1, Set=None):
        """Sets user avatar picture from file.

        :Parameters:
          Id : int
            Optional avatar Id.
          Set : str
            New avatar file name.

        :deprecated: Use `LoadAvatarFromFile` instead.
        """
        from warnings import warn
        warn('Settings.Avatar: Use Settings.LoadAvatarFromFile instead.', DeprecationWarning, stacklevel=2)
        if Set is None:
            raise TypeError('Argument \'Set\' is mandatory!')
        self.LoadAvatarFromFile(Set, Id)

    def LoadAvatarFromFile(self, Filename, AvatarId=1):
        """Loads user avatar picture from file.

        :Parameters:
          Filename : str
            Name of the avatar file.
          AvatarId : int
            Optional avatar Id.
        """
        s = 'AVATAR %s %s' % (AvatarId, path2unicode(Filename))
        self._Skype._DoCommand('SET %s' % s, s)

    def ResetIdleTimer(self):
        """Reset Skype idle timer.
        """
        self._Skype._DoCommand('RESETIDLETIMER')

    def RingTone(self, Id=1, Set=None):
        """Returns/sets a ringtone.

        :Parameters:
          Id : int
            Ringtone Id
          Set : str
            Path to new ringtone or None if the current path should be queried.

        :return: Current path if Set=None, None otherwise.
        :rtype: str or None
        """
        if Set is None:
            return unicode2path(self._Skype._Property('RINGTONE', Id, ''))
        self._Skype._Property('RINGTONE', Id, '', path2unicode(Set))

    def RingToneStatus(self, Id=1, Set=None):
        """Enables/disables a ringtone.

        :Parameters:
          Id : int
            Ringtone Id
          Set : bool
            True/False if the ringtone should be enabled/disabled or None if the current status
            should be queried.

        :return: Current status if Set=None, None otherwise.
        :rtype: bool
        """
        if Set is None:
            return (self._Skype._Property('RINGTONE', Id, 'STATUS') == 'ON')
        self._Skype._Property('RINGTONE', Id, 'STATUS', cndexp(Set, 'ON', 'OFF'))

    def SaveAvatarToFile(self, Filename, AvatarId=1):
        """Saves user avatar picture to file.

        :Parameters:
          Filename : str
            Destination path.
          AvatarId : int
            Avatar Id
        """
        s = 'AVATAR %s %s' % (AvatarId, path2unicode(Filename))
        self._Skype._DoCommand('GET %s' % s, s)

    def _Get_Skype(self):
        skype = self._SkypeRef()
        if skype:
            return skype
        raise ISkypeError('Skype4Py internal error')

    _Skype = property(_Get_Skype)

    def _GetAEC(self):
        return (self._Skype.Variable('AEC') == 'ON')

    def _SetAEC(self, Value):
        self._Skype.Variable('AEC', cndexp(Value, 'ON', 'OFF'))

    AEC = property(_GetAEC, _SetAEC,
    doc="""Automatic echo cancellation state.

    :type: bool

    :warning: Starting with Skype for Windows 3.6, this property has no effect. It can still be set
              for backwards compatibility reasons.
    """)

    def _GetAGC(self):
        return (self._Skype.Variable('AGC') == 'ON')

    def _SetAGC(self, Value):
        self._Skype.Variable('AGC', cndexp(Value, 'ON', 'OFF'))

    AGC = property(_GetAGC, _SetAGC,
    doc="""Automatic gain control state.

    :type: bool

    :warning: Starting with Skype for Windows 3.6, this property has no effect. It can still be set
              for backwards compatibility reasons.
    """)

    def _GetAudioIn(self):
        return self._Skype.Variable('AUDIO_IN')

    def _SetAudioIn(self, Value):
        self._Skype.Variable('AUDIO_IN', Value)

    AudioIn = property(_GetAudioIn, _SetAudioIn,
    doc="""Name of an audio input device.

    :type: unicode
    """)

    def _GetAudioOut(self):
        return self._Skype.Variable('AUDIO_OUT')

    def _SetAudioOut(self, Value):
        self._Skype.Variable('AUDIO_OUT', Value)

    AudioOut = property(_GetAudioOut, _SetAudioOut,
    doc="""Name of an audio output device.

    :type: unicode
    """)

    def _GetAutoAway(self):
        return (self._Skype.Variable('AUTOAWAY') == 'ON')

    def _SetAutoAway(self, Value):
        self._Skype.Variable('AUTOAWAY', cndexp(Value, 'ON', 'OFF'))

    AutoAway = property(_GetAutoAway, _SetAutoAway,
    doc="""Auto away status.

    :type: bool
    """)

    def _GetLanguage(self):
        return str(self._Skype.Variable('UI_LANGUAGE'))

    def _SetLanguage(self, Value):
        self._Skype.Variable('UI_LANGUAGE', Value)

    Language = property(_GetLanguage, _SetLanguage,
    doc="""Language of the Skype client as an ISO code.

    :type: str
    """)

    def _GetPCSpeaker(self):
        return (self._Skype.Variable('PCSPEAKER') == 'ON')

    def _SetPCSpeaker(self, Value):
        self._Skype.Variable('PCSPEAKER', cndexp(Value, 'ON', 'OFF'))

    PCSpeaker = property(_GetPCSpeaker, _SetPCSpeaker,
    doc="""PCSpeaker status.

    :type: bool
    """)

    def _GetRinger(self):
        return self._Skype.Variable('RINGER')

    def _SetRinger(self, Value):
        self._Skype.Variable('RINGER', Value)

    Ringer = property(_GetRinger, _SetRinger,
    doc="""Name of a ringer device.

    :type: unicode
    """)

    def _GetVideoIn(self):
        return self._Skype.Variable('VIDEO_IN')

    def _SetVideoIn(self, Value):
        self._Skype.Variable('VIDEO_IN', Value)

    VideoIn = property(_GetVideoIn, _SetVideoIn,
    doc="""Name of a video input device.

    :type: unicode
    """)



"""Language translations.
"""

import ar
import bg
import cs
import cz
import da
import de
import el
import en
import en
import es
import et
import fi
import fr
import he
import hu
import it
import ja
import ko
import lt
import lv
import nl
import no
import pl
import pl
import pp
import pt
import ro
import ru
import sv
import tr
import x1
